@@ -80,7 +80,7 @@ static void compareStrings(const T1 &e1, const T2 &e2)
     //qDebug() << "compareStrings():"
     //         << "\ne1:" << e1.toString() << e1.token() << e1.token().ToString()
     //         << "\ne2:" << e2.toString() << e2.token() << e2.token().toString();
-    QCOMPARE(e1.toString(0), e2.toString(0));
+    QCOMPARE(e1.toString(nullptr), e2.toString(nullptr));
     QCOMPARE(e1.token(), e2.token());
     QCOMPARE(e1.token().value(), e2.token().value());
     QCOMPARE(e1.token().toString(), e2.token().toString());
@@ -111,15 +111,15 @@ static void testCloneExpression(const T &e1)
 //! Validates expression @a expr and shows error message on failure
 static bool validate(KDbExpression *expr)
 {
-    KDbParseInfoInternal parseInfo(0);
+    KDbParseInfoInternal parseInfo(nullptr);
     bool ok = expr->validate(&parseInfo);
     if (!ok) {
-        qDebug() << "Validation of" << *expr << "FAILED.";
+        qInfo() << "Validation of" << *expr << "FAILED.";
         if (!parseInfo.errorMessage().isEmpty()) {
-            qDebug() << "Error message:" << parseInfo.errorMessage();
+            qInfo() << "Error message:" << parseInfo.errorMessage();
         }
         if (!parseInfo.errorDescription().isEmpty()) {
-            qDebug() << "Error description:" << parseInfo.errorDescription();
+            qInfo() << "Error description:" << parseInfo.errorDescription();
         }
     }
     return ok;
@@ -156,9 +156,9 @@ void ExpressionsTest::testNullExpression()
     e1 = e2;
     QVERIFY(e1.isNull());
     QCOMPARE(e1, e2);
-    QCOMPARE(e1.toString(0), KDbEscapedString("<UNKNOWN!>"));
+    QCOMPARE(e1.toString(nullptr), KDbEscapedString("<UNKNOWN!>"));
     QCOMPARE(e1.token().name(), QLatin1String("<INVALID_TOKEN>"));
-    QCOMPARE(e1.token().toString(0), QString("<INVALID_TOKEN>"));
+    QCOMPARE(e1.token().toString(nullptr), QString("<INVALID_TOKEN>"));
     compareStrings(e1, e2);
 
     KDbExpression e3(e2);
@@ -282,7 +282,7 @@ void ExpressionsTest::testNArgExpression()
     QCOMPARE(n.arg(1).toConst(), c2);
 
     QCOMPARE(n.token().name(), QString("+"));
-    QCOMPARE(n.toString(0), KDbEscapedString("7, 8"));
+    QCOMPARE(n.toString(nullptr), KDbEscapedString("7, 8"));
     n.setToken('*');
     QCOMPARE(n.token().name(), QString("*"));
 
@@ -467,7 +467,7 @@ void ExpressionsTest::testNArgExpression()
     n.append(KDbConstExpression(KDbToken::INTEGER_CONST, 1));
     n.append(KDbConstExpression(KDbToken::INTEGER_CONST, 2));
     n.append(KDbConstExpression(KDbToken::INTEGER_CONST, 3));
-    QCOMPARE(n.toString(0), KDbEscapedString("1, 2, 3"));
+    QCOMPARE(n.toString(nullptr), KDbEscapedString("1, 2, 3"));
     QCOMPARE(n.argCount(), 3);
     QVERIFY(!n.containsInvalidArgument());
     QVERIFY(!n.containsNullArgument());
@@ -480,7 +480,7 @@ void ExpressionsTest::testNArgExpression()
     QVERIFY(n.containsInvalidArgument());
     QVERIFY(!n.containsNullArgument());
     QVERIFY(!n.isNull());
-    QCOMPARE(n.toString(0), KDbEscapedString("1, <UNKNOWN!>, 3"));
+    QCOMPARE(n.toString(nullptr), KDbEscapedString("1, <UNKNOWN!>, 3"));
 
     // -- a list of arguments contains null argument
     n = KDbNArgExpression(KDb::ArgumentListExpression, ',');
@@ -489,7 +489,7 @@ void ExpressionsTest::testNArgExpression()
     n.prepend(KDbConstExpression(KDbToken::INTEGER_CONST, 0));
     QVERIFY(!n.containsInvalidArgument());
     QVERIFY(n.containsNullArgument());
-    QCOMPARE(n.toString(0), KDbEscapedString("0, 1, NULL"));
+    QCOMPARE(n.toString(nullptr), KDbEscapedString("0, 1, NULL"));
 }
 
 void ExpressionsTest::testUnaryExpression()
@@ -513,48 +513,48 @@ void ExpressionsTest::testUnaryExpression()
     u = KDbUnaryExpression('-', c1);
     testCloneExpression(u);
     QCOMPARE(u.token().name(), QString("-"));
-    QCOMPARE(u.toString(0), KDbEscapedString("-7"));
+    QCOMPARE(u.toString(nullptr), KDbEscapedString("-7"));
     QCOMPARE(c1, u.arg().toConst());
 
     u2 = KDbUnaryExpression('-', u);
     testCloneExpression(u);
     QCOMPARE(u2.token().name(), QString("-"));
-    QCOMPARE(u2.toString(0), KDbEscapedString("--7"));
+    QCOMPARE(u2.toString(nullptr), KDbEscapedString("--7"));
     QCOMPARE(u, u2.arg().toUnary());
 
     u = KDbUnaryExpression('(', c1);
     testCloneExpression(u);
-    QCOMPARE(u.toString(0), KDbEscapedString("(7)"));
+    QCOMPARE(u.toString(nullptr), KDbEscapedString("(7)"));
     QCOMPARE(c1, u.arg().toConst());
 
     c1 = KDbConstExpression(KDbToken::SQL_TRUE, true);
     u = KDbUnaryExpression(KDbToken::NOT, c1);
     testCloneExpression(u);
-    QCOMPARE(u.toString(0), KDbEscapedString("NOT TRUE"));
+    QCOMPARE(u.toString(nullptr), KDbEscapedString("NOT TRUE"));
     QCOMPARE(c1, u.arg().toConst());
 
     c1 = KDbConstExpression(KDbToken::SQL_NULL, QVariant());
     u = KDbUnaryExpression(KDbToken::NOT, c1);
     testCloneExpression(u);
-    QCOMPARE(u.toString(0), KDbEscapedString("NOT NULL"));
+    QCOMPARE(u.toString(nullptr), KDbEscapedString("NOT NULL"));
     QCOMPARE(c1, u.arg().toConst());
 
     c1 = KDbConstExpression(KDbToken::SQL_NULL, QVariant());
     u = KDbUnaryExpression(KDbToken::SQL_IS_NULL, c1);
     testCloneExpression(u);
-    QCOMPARE(u.toString(0), KDbEscapedString("NULL IS NULL"));
+    QCOMPARE(u.toString(nullptr), KDbEscapedString("NULL IS NULL"));
     QCOMPARE(c1, u.arg().toConst());
 
     c1 = KDbConstExpression(KDbToken::SQL_NULL, QVariant());
     u = KDbUnaryExpression(KDbToken::SQL_IS_NOT_NULL, c1);
     testCloneExpression(u);
-    QCOMPARE(u.toString(0), KDbEscapedString("NULL IS NOT NULL"));
+    QCOMPARE(u.toString(nullptr), KDbEscapedString("NULL IS NOT NULL"));
     QCOMPARE(c1, u.arg().toConst());
 
     c1 = KDbConstExpression(KDbToken::INTEGER_CONST, 17);
     u = KDbUnaryExpression(KDbToken::SQL, c1);
     testCloneExpression(u);
-    QCOMPARE(u.toString(0), KDbEscapedString("SQL 17"));
+    QCOMPARE(u.toString(nullptr), KDbEscapedString("SQL 17"));
     QCOMPARE(c1, u.arg().toConst());
 
     // -- exchanging arg between two unary expressions
@@ -580,8 +580,16 @@ void ExpressionsTest::testUnaryExpression()
     u2 = KDbUnaryExpression('+', c1);
     u2.setArg(u);
     u.setArg(u2);
-    QCOMPARE(u.toString(0), KDbEscapedString("-+<CYCLE!>"));
-    QCOMPARE(u2.toString(0), KDbEscapedString("+-<CYCLE!>"));
+
+    QTest::ignoreMessage(QtWarningMsg, R"w(Cycle detected in expression (depth 2):
+1: Unary -
+2: Unary +)w");
+    QCOMPARE(u.toString(nullptr), KDbEscapedString("-+<CYCLE!>"));
+
+    QTest::ignoreMessage(QtWarningMsg, R"w(Cycle detected in expression (depth 2):
+1: Unary +
+2: Unary -)w");
+    QCOMPARE(u2.toString(nullptr), KDbEscapedString("+-<CYCLE!>"));
 }
 
 void ExpressionsTest::testBinaryExpression()
@@ -599,25 +607,31 @@ void ExpressionsTest::testBinaryExpression()
     QVERIFY(emptyBinary.left().isNull());
     QVERIFY(emptyBinary.right().isNull());
 
+    QTest::ignoreMessage(QtWarningMsg,
+                         "Setting KDbBinaryExpression to null because left argument is not specified");
     b = KDbBinaryExpression(KDbExpression(), '-', KDbExpression());
     QVERIFY(b.left().isNull());
     QVERIFY(b.right().isNull());
     QVERIFY(b.isNull()); // it's null because args are null
-    qDebug() << b.toString(0);
-    QCOMPARE(b.toString(0), KDbEscapedString("<UNKNOWN!>"));
+    //qDebug() << b.toString(nullptr);
+    QCOMPARE(b.toString(nullptr), KDbEscapedString("<UNKNOWN!>"));
     c = KDbConstExpression(KDbToken::INTEGER_CONST, 10);
+    QTest::ignoreMessage(QtWarningMsg,
+                         "Setting KDbBinaryExpression to null because right argument is not specified");
     b = KDbBinaryExpression(c, '-', KDbExpression());
     QVERIFY(b.left().isNull());
     QVERIFY(b.right().isNull());
     QVERIFY(b.isNull()); // it's null because one arg is null
-    qDebug() << b.toString(0);
-    QCOMPARE(b.toString(0), KDbEscapedString("<UNKNOWN!>"));
+    //qDebug() << b.toString(nullptr);
+    QCOMPARE(b.toString(nullptr), KDbEscapedString("<UNKNOWN!>"));
+    QTest::ignoreMessage(QtWarningMsg,
+                         "Setting KDbBinaryExpression to null because left argument is not specified");
     b = KDbBinaryExpression(KDbExpression(), '-', c);
     QVERIFY(b.left().isNull());
     QVERIFY(b.right().isNull());
     QVERIFY(b.isNull()); // it's null because one arg is null
-    qDebug() << b.toString(0);
-    QCOMPARE(b.toString(0), KDbEscapedString("<UNKNOWN!>"));
+    //qDebug() << b.toString(nullptr);
+    QCOMPARE(b.toString(nullptr), KDbEscapedString("<UNKNOWN!>"));
 
     // -- copy ctor & cloning
     c = KDbConstExpression(KDbToken::INTEGER_CONST, 3);
@@ -625,53 +639,58 @@ void ExpressionsTest::testBinaryExpression()
     b = KDbBinaryExpression(c, '/', c1);
     testCloneExpression(b);
     QCOMPARE(b.token().name(), QString("/"));
-    QCOMPARE(b.toString(0), KDbEscapedString("3 / 4"));
+    QCOMPARE(b.toString(nullptr), KDbEscapedString("3 / 4"));
     QCOMPARE(c1, b.right().toConst());
 
     b2 = KDbBinaryExpression(b, '*', b.clone());
     testCloneExpression(b2);
     QCOMPARE(b2.token().name(), QString("*"));
-    QCOMPARE(b2.toString(0), KDbEscapedString("3 / 4 * 3 / 4"));
+    QCOMPARE(b2.toString(nullptr), KDbEscapedString("3 / 4 * 3 / 4"));
     QCOMPARE(b, b2.left().toBinary());
 
     // -- cycles
     // --- ref to parent
     b = KDbBinaryExpression(
             KDbConstExpression(KDbToken::INTEGER_CONST, 1), '+', KDbConstExpression(KDbToken::INTEGER_CONST, 2));
-    KDbEscapedString s = b.toString(0);
+    KDbEscapedString s = b.toString(nullptr);
+    QTest::ignoreMessage(QtWarningMsg,
+                         QRegularExpression("Expression BinaryExp(.*) cannot be set as own child"));
     b.setLeft(b); // should not work
-    qDebug() << b.toString(0);
-    QCOMPARE(s, b.toString(0));
+    //qDebug() << b.toString(nullptr);
+    QCOMPARE(s, b.toString(nullptr));
     // --- cannot set twice
     c = b.left().toConst();
     b.setLeft(c);
-    QCOMPARE(s, b.toString(0));
+    QCOMPARE(s, b.toString(nullptr));
     // --- ref to grandparent
     b = KDbBinaryExpression(
             KDbConstExpression(KDbToken::INTEGER_CONST, 1), '+', KDbConstExpression(KDbToken::INTEGER_CONST, 2));
     c = KDbConstExpression(KDbToken::INTEGER_CONST, 10);
     b2 = KDbBinaryExpression(b, '-', c);
-    qDebug() << b2.toString(0);
-    QCOMPARE(b2.toString(0), KDbEscapedString("1 + 2 - 10"));
+    //qDebug() << b2.toString(nullptr);
+    QCOMPARE(b2.toString(nullptr), KDbEscapedString("1 + 2 - 10"));
+    QTest::ignoreMessage(QtWarningMsg, R"w(Cycle detected in expression (depth 2):
+1: Arithm -
+2: Arithm +)w");
     b.setRight(b2);
-    qDebug() << b2.toString(0);
-    QCOMPARE(b2.toString(0), KDbEscapedString("1 + <CYCLE!> - 10"));
+    //qDebug() << b2.toString(nullptr);
+    QCOMPARE(b2.toString(nullptr), KDbEscapedString("1 + <CYCLE!> - 10"));
 
     // -- moving right argument to left should remove right arg
     b = KDbBinaryExpression(
             KDbConstExpression(KDbToken::INTEGER_CONST, 1), '+', KDbConstExpression(KDbToken::INTEGER_CONST, 2));
     c = b.right().toConst();
     b.setLeft(c);
-    qDebug() << b.toString(0);
-    QCOMPARE(b.toString(0), KDbEscapedString("2 + <UNKNOWN!>"));
+    //qDebug() << b.toString(nullptr);
+    QCOMPARE(b.toString(nullptr), KDbEscapedString("2 + <UNKNOWN!>"));
 
     // -- moving left argument to right should remove left arg
     b = KDbBinaryExpression(
             KDbConstExpression(KDbToken::INTEGER_CONST, 1), '+', KDbConstExpression(KDbToken::INTEGER_CONST, 2));
     c = b.left().toConst();
     b.setRight(c);
-    qDebug() << b.toString(0);
-    QCOMPARE(b.toString(0), KDbEscapedString("<UNKNOWN!> + 1"));
+    //qDebug() << b.toString(nullptr);
+    QCOMPARE(b.toString(nullptr), KDbEscapedString("<UNKNOWN!> + 1"));
 }
 
 void ExpressionsTest::testBinaryExpressionCloning_data()
@@ -723,7 +742,7 @@ void ExpressionsTest::testBinaryExpressionCloning()
     QCOMPARE(b.token(), token);
     QCOMPARE(b.token().name(), token.name());
     //qDebug() << token << b;
-    QCOMPARE(b.toString(0), KDbEscapedString(string));
+    QCOMPARE(b.toString(nullptr), KDbEscapedString(string));
     QCOMPARE(c, b.left().toConst());
     QCOMPARE(c1, b.right().toConst());
 }
@@ -754,7 +773,7 @@ void ExpressionsTest::testFunctionExpression()
     //qDebug() << f_substr.toString();
     //qDebug() << f_substr2.toString();
     QVERIFY(f_substr != f_substr2); // other objects
-    QCOMPARE(f_substr.toString(0), f_substr2.toString(0)); // the same signatures
+    QCOMPARE(f_substr.toString(nullptr), f_substr2.toString(nullptr)); // the same signatures
     QCOMPARE(f_substr.arguments(), f_substr2.arguments()); // the same arg lists
 
     // clone the args
@@ -762,7 +781,7 @@ void ExpressionsTest::testFunctionExpression()
     //qDebug() << f_substr2;
     f_substr2.setArguments(args2);
     //qDebug() << f_substr2;
-    QCOMPARE(f_substr.toString(0), f_substr2.toString(0)); // still the same signatures
+    QCOMPARE(f_substr.toString(nullptr), f_substr2.toString(nullptr)); // still the same signatures
     QVERIFY(f_substr.arguments() != f_substr2.arguments()); // not the same arg lists
 
     KDbExpression e = f_substr;
@@ -790,7 +809,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.type(), KDbField::Null);
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     // integer
     c = KDbConstExpression(KDbToken::INTEGER_CONST, -0x7f);
@@ -806,7 +825,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(-0x80));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, -10);
     QCOMPARE(c.type(), KDbField::Byte);
@@ -815,7 +834,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(-10));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, 0);
     QCOMPARE(c.type(), KDbField::Byte);
@@ -824,7 +843,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(0));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, 20);
     QCOMPARE(c.type(), KDbField::Byte);
@@ -833,7 +852,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(20));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, 255);
     QCOMPARE(c.type(), KDbField::Byte);
@@ -842,7 +861,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(255));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, -0x80);
     QCOMPARE(c.type(), KDbField::ShortInteger);
@@ -851,7 +870,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(-0x80));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, -0x7fff);
     QCOMPARE(c.type(), KDbField::ShortInteger);
@@ -860,7 +879,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(-0x7fff));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, 256);
     QCOMPARE(c.type(), KDbField::ShortInteger);
@@ -869,7 +888,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(256));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, 0xffff);
     QCOMPARE(c.type(), KDbField::ShortInteger);
@@ -878,7 +897,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(0xffff));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, -0x8000);
     QCOMPARE(c.type(), KDbField::Integer);
@@ -887,7 +906,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(-0x8000));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, uint(0x10000));
     QCOMPARE(c.type(), KDbField::Integer);
@@ -896,7 +915,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(0x10000));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, qlonglong(-0x100000));
     QCOMPARE(c.type(), KDbField::BigInteger);
@@ -905,7 +924,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(-0x100000));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::INTEGER_CONST, qulonglong(0x1000000));
     QCOMPARE(c.type(), KDbField::BigInteger);
@@ -914,7 +933,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(0x1000000));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     // string
     int oldMaxLen = KDbField::defaultMaxLength(); // save
@@ -926,7 +945,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant("01234567890"));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     KDbField::setDefaultMaxLength(10);
     c = KDbConstExpression(KDbToken::CHARACTER_STRING_LITERAL, QString());
@@ -936,7 +955,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(QString()));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::CHARACTER_STRING_LITERAL, QVariant());
     QCOMPARE(c.type(), KDbField::Text);
@@ -945,7 +964,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant());
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::CHARACTER_STRING_LITERAL, "01234567890");
     QCOMPARE(c.type(), KDbField::LongText);
@@ -953,13 +972,13 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(c.isTextType());
     QCOMPARE(c.value(), QVariant("01234567890"));
     QVERIFY(validate(&c));
-    qDebug() << c;
+    //qDebug() << c;
     c.setValue("ąćę");
     QCOMPARE(c.value(), QVariant("ąćę"));
     QCOMPARE(c.type(), KDbField::Text);
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     KDbField::setDefaultMaxLength(oldMaxLen); // restore
 
@@ -971,12 +990,12 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(!c.isNumericType());
     QCOMPARE(c.value(), QVariant(true));
     QVERIFY(validate(&c));
-    qDebug() << c;
+    //qDebug() << c;
     c.setValue(false);
     QCOMPARE(c.value(), QVariant(false));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::SQL_FALSE, false);
     QCOMPARE(c.type(), KDbField::Boolean);
@@ -986,7 +1005,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(false));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     // real
     c = KDbConstExpression(KDbToken::REAL_CONST, QVariant());
@@ -995,10 +1014,10 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(c.isNumericType());
     QVERIFY(c.isFPNumericType());
     QCOMPARE(c.value(), QVariant());
-    QCOMPARE(c.toString(0), KDbEscapedString());
+    QCOMPARE(c.toString(nullptr), KDbEscapedString());
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     c = KDbConstExpression(KDbToken::REAL_CONST, 3.14159);
     QCOMPARE(c.type(), KDbField::Double);
@@ -1008,15 +1027,15 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.value(), QVariant(3.14159));
     QString piString("3.14159");
     // limit precision because it depends on the OS
-    QCOMPARE(c.toString(0).toString().left(piString.length() - 1), piString.left(piString.length() - 1));
+    QCOMPARE(c.toString(nullptr).toString().left(piString.length() - 1), piString.left(piString.length() - 1));
     QVERIFY(validate(&c));
-    qDebug() << c;
+    //qDebug() << c;
     c.setValue(-18.012);
     QCOMPARE(c.value(), QVariant(-18.012));
-    QCOMPARE(c.toString(0), KDbEscapedString("-18.012"));
+    QCOMPARE(c.toString(nullptr), KDbEscapedString("-18.012"));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     QByteArray largeDecimal("2147483647.2147483647");
     c = KDbConstExpression(KDbToken::REAL_CONST, largeDecimal);
@@ -1025,17 +1044,17 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(c.isNumericType());
     QVERIFY(c.isFPNumericType());
     QCOMPARE(c.value(), QVariant(largeDecimal));
-    QCOMPARE(c.toString(0), KDbEscapedString(largeDecimal));
+    QCOMPARE(c.toString(nullptr), KDbEscapedString(largeDecimal));
     largeDecimal = "-10.2147483647";
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
     c = KDbConstExpression(KDbToken::REAL_CONST, largeDecimal);
     QCOMPARE(c.value(), QVariant(largeDecimal));
-    QCOMPARE(c.toString(0), KDbEscapedString(largeDecimal));
+    QCOMPARE(c.toString(nullptr), KDbEscapedString(largeDecimal));
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     // date
     QDate date(QDate::currentDate());
@@ -1045,7 +1064,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QCOMPARE(c.type(), KDbField::Date);
     QCOMPARE(c.value(), QVariant(date));
     QVERIFY(validate(&c));
-    qDebug() << c;
+    //qDebug() << c;
     date = date.addDays(17);
     c.setValue(date);
     QCOMPARE(c.value(), QVariant(date));
@@ -1053,7 +1072,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(c.isDateTimeType());
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     // date/time
     QDateTime dateTime(QDateTime::currentDateTime());
@@ -1063,7 +1082,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(c.isDateTimeType());
     QCOMPARE(c.value(), QVariant(dateTime));
     QVERIFY(validate(&c));
-    qDebug() << c;
+    //qDebug() << c;
     dateTime = dateTime.addDays(-17);
     c.setValue(dateTime);
     QCOMPARE(c.value(), QVariant(dateTime));
@@ -1071,7 +1090,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(c.isDateTimeType());
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     // time
     QTime time(QTime::currentTime());
@@ -1080,7 +1099,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(c.isValid());
     QVERIFY(c.isDateTimeType());
     QCOMPARE(c.value(), QVariant(time));
-    qDebug() << c;
+    //qDebug() << c;
     QVERIFY(validate(&c));
     time = time.addSecs(1200);
     c.setValue(time);
@@ -1089,7 +1108,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(c.isDateTimeType());
     QVERIFY(validate(&c));
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 
     // setValue()
     c = KDbConstExpression(KDbToken::INTEGER_CONST, 124);
@@ -1100,7 +1119,7 @@ void ExpressionsTest::testConstExpressionValidate()
     QVERIFY(c.isNumericType());
     QVERIFY(!c.isFPNumericType());
     testCloneExpression(c);
-    qDebug() << c;
+    //qDebug() << c;
 }
 
 void ExpressionsTest::testUnaryExpressionValidate()
@@ -1117,8 +1136,21 @@ void ExpressionsTest::testUnaryExpressionValidate()
     u2 = KDbUnaryExpression('+', c1);
     u2.setArg(u);
     u.setArg(u2);
+    const char *warning = R"w(Cycle detected in expression (depth 2):
+1: Unary -
+2: Unary +)w";
+    QTest::ignoreMessage(QtWarningMsg, warning);
+    warning = R"w(Cycle detected in expression (depth 2):
+1: Unary +
+2: Unary -)w";
+    QTest::ignoreMessage(QtWarningMsg, warning);
+    warning = R"w(Cycle detected in expression (depth 2):
+1: Unary -
+2: Unary +)w";
+    QTest::ignoreMessage(QtWarningMsg, warning);
+    QTest::ignoreMessage(QtWarningMsg, warning);
     QVERIFY(!validate(&u));
-    //qDebug() << c << u << c1 << u2;
+    ////qDebug() << c << u << c1 << u2;
 
     // NOT NULL is NULL
     c = KDbConstExpression(KDbToken::SQL_NULL, QVariant());
@@ -1154,7 +1186,7 @@ void ExpressionsTest::testNArgExpressionValidate()
     QCOMPARE(n.type(), KDbField::Tuple);
     QVERIFY(validate(&n));
     testCloneExpression(n);
-    //qDebug() << c << c1 << n;
+    ////qDebug() << c << c1 << n;
 
     // -- a list of arguments
     n = KDbNArgExpression(KDb::ArgumentListExpression, ',');
@@ -1180,29 +1212,38 @@ void ExpressionsTest::testBinaryExpressionValidate_data()
 
     // invalid
     KDbConstExpression c(KDbToken::INTEGER_CONST, 7);
+    QTest::ignoreMessage(QtWarningMsg,
+                         "Setting KDbBinaryExpression to null because right argument is not specified");
     KDbBinaryExpression b(c, '+', KDbExpression());
     QCOMPARE(b.type(), KDbField::InvalidType);
     QVERIFY(!validate(&b));
     testCloneExpression(b);
-    qDebug() << b;
+    //qDebug() << b;
 
+    QTest::ignoreMessage(QtWarningMsg,
+                         "Setting KDbBinaryExpression to null because left argument is not specified");
     b = KDbBinaryExpression(KDbExpression(), '/', KDbExpression());
     QCOMPARE(b.type(), KDbField::InvalidType);
     QVERIFY(!validate(&b)); // unknown class
     testCloneExpression(b);
-    qDebug() << b;
+    //qDebug() << b;
 
     // invalid left or right
+    QTest::ignoreMessage(QtWarningMsg,
+                         "Setting KDbBinaryExpression to null because left argument is not specified");
     KDbBinaryExpression b2(b, '*', c.clone());
     QCOMPARE(b2.type(), KDbField::InvalidType);
     QVERIFY(!validate(&b2)); // unknown class
     testCloneExpression(b2);
-    qDebug() << b2;
+    //qDebug() << b2;
+
+    QTest::ignoreMessage(QtWarningMsg,
+                         "Setting KDbBinaryExpression to null because right argument is not specified");
     KDbBinaryExpression b3(c.clone(), '*', b);
     QCOMPARE(b3.type(), KDbField::InvalidType);
     QVERIFY(!validate(&b3)); // unknown class
     testCloneExpression(b3);
-    qDebug() << b3;
+    //qDebug() << b3;
 
 #define TNAME(type) type.name().toLatin1()
 
@@ -1355,8 +1396,8 @@ void ExpressionsTest::testBinaryExpressionValidate()
     KDbConstExpression c(type1, const1);
     KDbConstExpression c1(type2, const2);
     KDbBinaryExpression b(c, token, c1);
-    qDebug() << b.type();
-    qDebug() << type3;
+    //qDebug() << b.type();
+    //qDebug() << type3;
     QCOMPARE(b.type(), type3);
     QVERIFY(validate(&b) == (type3 != KDbField::InvalidType));
     testCloneExpression(b);
