@@ -20,23 +20,16 @@
 #include "KDbTest.h"
 
 #include <KDb>
+#include <KDbConnectionData>
 #include <KDbVersionInfo>
 
+#include <QRegularExpression>
 #include <QTest>
 
 QTEST_GUILESS_MAIN(KDbTest)
 
-Q_DECLARE_METATYPE(KDbField::TypeGroup)
-Q_DECLARE_METATYPE(KDbField::Type)
-Q_DECLARE_METATYPE(KDb::Signedness)
-Q_DECLARE_METATYPE(QList<KDbField::Type>)
-Q_DECLARE_METATYPE(KDb::BLOBEscapingType)
-
 void KDbTest::initTestCase()
 {
-    QVERIFY(utils.testDriverManager());
-    //! @todo don't hardcode SQLite here
-    QVERIFY(utils.testSqliteDriver());
 }
 
 void KDbTest::testVersionInfo()
@@ -282,37 +275,6 @@ KDB_EXPORT KDbEscapedString sqlWhere(KDbDriver *drv, KDbField::Type t,
 KDB_EXPORT tristate idForObjectName(KDbConnection* conn, int *id, const QString& objName,
                                     int objType);
 
-//! @todo perhaps use quint64 here?
-/*! @return number of records that can be retrieved after executing @a sql statement
- within a connection @a conn. The statement should be of type SELECT.
- For SQL data sources it does not fetch any records, only "COUNT(*)"
- SQL aggregation is used at the backed.
- -1 is returned if error occurred. */
-KDB_EXPORT int recordCount(KDbConnection* conn, const KDbEscapedString& sql);
-
-//! @todo perhaps use quint64 here?
-/*! @return number of records that can be retrieved from @a tableSchema.
- The table must be created or retrieved using a KDbConnection object,
- i.e. tableSchema.connection() must not return 0.
- For SQL data sources it does not fetch any records, only "COUNT(*)"
- SQL aggregation is used at the backed.
- -1 is returned if error occurred. */
-KDB_EXPORT int recordCount(const KDbTableSchema& tableSchema);
-
-/*! @overload in rowCount(const KDbTableSchema& tableSchema)
- Operates on a query schema. @a params are optional values of parameters that will
- be inserted into places marked with [] before execution of the query. */
-//! @todo perhaps use quint64 here?
-KDB_EXPORT int recordCount(KDbQuerySchema* querySchema,
-                           const QList<QVariant>& params = QList<QVariant>());
-
-/*! @overload int rowCount(KDbQuerySchema& querySchema, const QList<QVariant>& params)
- Operates on a table or query schema. @a params are optional values of parameters that
- will be inserted into places marked with [] before execution of the query. */
-//! @todo perhaps use quint64 here?
-KDB_EXPORT int recordCount(KDbTableOrQuerySchema* tableOrQuery,
-                           const QList<QVariant>& params = QList<QVariant>());
-
 /*! @return a number of columns that can be retrieved from table or query schema.
  In case of query, expanded fields are counted. Can return -1 if @a tableOrQuery
  has neither table or query assigned. */
@@ -475,7 +437,7 @@ KDB_EXPORT QVariant notEmptyValueForFieldType(KDbField::Type type);
 /*! @return true if the @a word is an reserved KDbSQL keyword
  See src/generated/sqlkeywords.cpp in the KDb source code.
  @todo add function returning list of keywords. */
-KDB_EXPORT bool isKDbSQLKeyword(const QByteArray& word);
+KDB_EXPORT bool isKDbSqlKeyword(const QByteArray& word);
 
 //! @return @a string string with applied KDbSQL identifier escaping
 /*! This escaping can be used for field, table, database names, etc.
@@ -501,6 +463,22 @@ KDB_EXPORT QByteArray escapeIdentifierAndAddQuotes(const QByteArray& string);
     Also escapes \\n, \\r, \\t, \\\\, \\0.
     Use it for user-visible backend-independent statements. */
 KDB_EXPORT QString escapeString(const QString& string);
+
+/**
+ * @brief Returns escaped string @a string
+ *
+ * If @a drv driver is present, it is used to perform escaping, otherwise escapeString() is used
+ * so the KDbSQL dialect-escaping is performed.
+ */
+KDB_EXPORT KDbEscapedString escapeString(KDbDriver *drv, const QString& string);
+
+/**
+ * @brief Returns escaped string @a string
+ *
+ * If @a conn is present, its driver is used to perform escaping, otherwise escapeString() is used
+ * so the KDbSQL dialect-escaping is performed.
+ */
+KDB_EXPORT KDbEscapedString escapeString(KDbConnection *conn, const QString& string);
 #endif
 
 void KDbTest::testUnescapeString_data()
@@ -639,11 +617,11 @@ void KDbTest::testUnescapeStringHelper(const QString &sequenceString, const QStr
     if (quote == 0) { // both cases
         COMPARE(KDb::unescapeString(sequenceString, '\'', &actualErrorPosition), resultString);
         COMPARE(actualErrorPosition, errorPosition);
-        COMPARE(KDb::unescapeString(sequenceString, '\'', 0), resultString);
+        COMPARE(KDb::unescapeString(sequenceString, '\'', nullptr), resultString);
 
         COMPARE(KDb::unescapeString(sequenceString, '"', &actualErrorPosition), resultString);
         COMPARE(actualErrorPosition, errorPosition);
-        COMPARE(KDb::unescapeString(sequenceString, '"', 0), resultString);
+        COMPARE(KDb::unescapeString(sequenceString, '"', nullptr), resultString);
     } else {
         if (quote != '\'' && quote != '"') {
             resultString.clear();
@@ -651,7 +629,7 @@ void KDbTest::testUnescapeStringHelper(const QString &sequenceString, const QStr
         }
         COMPARE(KDb::unescapeString(sequenceString, quote, &actualErrorPosition), resultString);
         COMPARE(actualErrorPosition, errorPosition);
-        COMPARE(KDb::unescapeString(sequenceString, quote, 0), resultString);
+        COMPARE(KDb::unescapeString(sequenceString, quote, nullptr), resultString);
     }
 #undef CHECK_POS
 }
@@ -701,16 +679,16 @@ void KDbTest::testEscapeBLOB()
     QFETCH(QString, escapedOctal);
     QFETCH(QString, escapedBytea);
 
-    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscapeXHex), escapedX);
-    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscape0xHex), escaped0x);
-    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscapeHex), escapedHex);
-    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscapeOctal), escapedOctal);
-    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscapeByteaHex), escapedBytea);
+    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscapingType::XHex), escapedX);
+    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscapingType::ZeroXHex), escaped0x);
+    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscapingType::Hex), escapedHex);
+    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscapingType::Octal), escapedOctal);
+    QCOMPARE(KDb::escapeBLOB(blob, KDb::BLOBEscapingType::ByteaHex), escapedBytea);
 }
 
 void KDbTest::testPgsqlByteaToByteArray()
 {
-    QCOMPARE(KDb::pgsqlByteaToByteArray(0, 0), QByteArray());
+    QCOMPARE(KDb::pgsqlByteaToByteArray(nullptr, 0), QByteArray());
     QCOMPARE(KDb::pgsqlByteaToByteArray("", 0), QByteArray());
     QCOMPARE(KDb::pgsqlByteaToByteArray(" ", 0), QByteArray());
     QCOMPARE(KDb::pgsqlByteaToByteArray("\\101"), QByteArray("A"));
@@ -751,7 +729,7 @@ void KDbTest::testXHexToByteArray()
     bool actualOk;
     QCOMPARE(KDb::xHexToByteArray(data.constData(), length == -1 ? data.length() : length, &actualOk), result);
     QCOMPARE(actualOk, ok);
-    QCOMPARE(KDb::xHexToByteArray(data.constData(), length, 0), result);
+    QCOMPARE(KDb::xHexToByteArray(data.constData(), length, nullptr), result);
 }
 
 void KDbTest::testZeroXHexToByteArray_data()
@@ -785,7 +763,7 @@ void KDbTest::testZeroXHexToByteArray()
     bool actualOk;
     QCOMPARE(KDb::zeroXHexToByteArray(data.constData(), length == -1 ? data.length() : length, &actualOk), result);
     QCOMPARE(actualOk, ok);
-    QCOMPARE(KDb::zeroXHexToByteArray(data.constData(), length, 0), result);
+    QCOMPARE(KDb::zeroXHexToByteArray(data.constData(), length, nullptr), result);
 }
 
 //! @todo add tests
@@ -940,15 +918,15 @@ void KDbTest::testCstringToVariant()
     QFETCH(bool, okResult);
     bool ok;
     const QByteArray ba(data.toUtf8()); // to avoid pointer to temp.
-    const char *realData = ba.isNull() ? 0 : ba.constData();
+    const char *realData = ba.isNull() ? nullptr : ba.constData();
     QCOMPARE(KDb::cstringToVariant(realData, type, &ok, length, signedness), variant);
     QCOMPARE(ok, okResult);
-    QCOMPARE(KDb::cstringToVariant(realData, type, 0, length, signedness), variant); // a case where ok == 0
+    QCOMPARE(KDb::cstringToVariant(realData, type, nullptr, length, signedness), variant); // a case where ok == 0
     if (realData) {
         QCOMPARE(KDb::cstringToVariant(realData, type, &ok, data.length(), signedness), variant); // a case where length is set
         QCOMPARE(ok, okResult);
     }
-    QCOMPARE(KDb::cstringToVariant(0, type, &ok, length, signedness), QVariant()); // a case where data == 0 (NULL)
+    QCOMPARE(KDb::cstringToVariant(nullptr, type, &ok, length, signedness), QVariant()); // a case where data == 0 (NULL)
     QVERIFY(ok || type < KDbField::Byte || type > KDbField::LastType); // fails for NULL if this type isn't allowed
     if (type != KDbField::Boolean) {
         QCOMPARE(KDb::cstringToVariant(realData, type, &ok, 0, signedness), QVariant()); // a case where length == 0
@@ -976,16 +954,16 @@ KDB_EXPORT QString defaultFileBasedDriverId();
 
 /*! Escapes and converts value @a v (for type @a ftype)
     to string representation required by KDbSQL commands.
-    For Date/Time type KDb::dateTimeToSQL() is used.
-    For BLOB type KDb::escapeBlob() with BLOBEscape0xHex conversion type is used. */
-KDB_EXPORT KDbEscapedString valueToSQL(KDbField::Type ftype, const QVariant& v);
+    For Date/Time type KDb::dateTimeToSql() is used.
+    For BLOB type KDb::escapeBlob() with BLOBEscapingType::ZeroXHex conversion type is used. */
+KDB_EXPORT KDbEscapedString valueToSql(KDbField::Type ftype, const QVariant& v);
 
 /*! Converts value @a v to string representation required by KDbSQL commands:
     ISO 8601 DateTime format - with "T" delimiter/
     For specification see http://www.w3.org/TR/NOTE-datetime.
     Example: "1994-11-05T13:15:30" not "1994-11-05 13:15:30".
     @todo Add support for time zones */
-KDB_EXPORT KDbEscapedString dateTimeToSQL(const QDateTime& v);
+KDB_EXPORT KDbEscapedString dateTimeToSql(const QDateTime& v);
 
 #ifdef KDB_DEBUG_GUI
 //! A prototype of handler for GUI debugger
@@ -1053,12 +1031,7 @@ KDB_EXPORT QStringList libraryPaths();
 
 void KDbTest::testTemporaryTableName()
 {
-    QVERIFY(utils.driver);
-    QString dbName(QDir::fromNativeSeparators(QFile::decodeName(FILES_OUTPUT_DIR "/KDbTest.kexi")));
-    QVERIFY(utils.testCreate(dbName));
-    QVERIFY(utils.connection);
-    utils.connection->useDatabase();
-    QVERIFY(utils.testCreateTables());
+    QVERIFY(utils.testCreateDbWithTables("KDbTest"));
 
     QString baseName = QLatin1String("foobar");
     QString tempName1 = KDb::temporaryTableName(utils.connection.data(), baseName);
@@ -1068,13 +1041,20 @@ void KDbTest::testTemporaryTableName()
     QVERIFY(!tempName2.isEmpty());
     QVERIFY(tempName2.contains(baseName));
     QVERIFY(tempName1 != tempName2);
+
     utils.connection->closeDatabase();
+    QTest::ignoreMessage(QtWarningMsg, "Missing database handle");
+    QTest::ignoreMessage(QtWarningMsg, QRegularExpression("!executeQuery().*"));
     QString tempName = KDb::temporaryTableName(utils.connection.data(), baseName);
-    QVERIFY2(tempName.isEmpty(), "Temporary name should not be created for closed connection");
+    QVERIFY2(tempName.isEmpty(), "Temporary name should not be created when database is closed ");
+
     utils.connection->disconnect();
+    QTest::ignoreMessage(QtWarningMsg, "Missing database handle");
+    QTest::ignoreMessage(QtWarningMsg, QRegularExpression("!executeQuery().*"));
     tempName = KDb::temporaryTableName(utils.connection.data(), baseName);
-    QVERIFY2(tempName.isEmpty(), "Temporary name should not be created for closed connection");
-    utils.connection->dropDatabase(dbName);
+    QVERIFY2(tempName.isEmpty(), "Temporary name should not be created connection is missing");
+
+    utils.connection->dropDatabase(utils.connection->data().databaseName());
 }
 
 //! @todo add tests
@@ -1089,7 +1069,7 @@ KDB_EXPORT QString sqlite3ProgramPath();
  @return true on success. */
 KDB_EXPORT bool importSqliteFile(const QString &inputFileName, const QString &outputFileName);
 
-/*! @return true if @a s is a valid identifier, ie. starts with a letter or '_' character
+/*! @return @c true if @a s is a valid identifier, i.e. starts with a letter or '_' character
  and contains only letters, numbers and '_' character. */
 KDB_EXPORT bool isIdentifier(const QString& s);
 
@@ -1105,6 +1085,125 @@ KDB_EXPORT QString identifierExpectedMessage(const QString &valueName,
         const QVariant& v);
 #endif
 
+void KDbTest::deleteRecordWithOneConstraintsTest()
+{
+    QVERIFY(utils.testCreateDbWithTables("KDbTest"));
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "id", 2));
+    QVERIFY2(KDb::deleteRecords(utils.connection.data(), "persons", "id", "3"),
+            "Passing a valid Integer in String Format");
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "id", "Foo"));
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "name", "Jaroslaw"));
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "surname", "FooBar"));
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "age", 45));
+    // and empty data.
+    KDbTableSchema *kdb_t = utils.connection.data()->tableSchema("persons");
+    QVERIFY(kdb_t);
+    QVERIFY2(utils.connection.data()->insertRecord(kdb_t, 10, 20, QVariant(), "Bar"),
+             "Inserting NULL data");
+    QVERIFY2(utils.connection.data()->insertRecord(kdb_t,15, 20, "", "Bar"),
+             "Inserting empty data");
+    QVERIFY2(KDb::deleteRecords(utils.connection.data(), "persons", "name", QString()),
+            "Passing a null value instead of string");
+    //
+    QVERIFY2(KDb::deleteRecords(utils.connection.data(), "persons", "name", ""),
+             "Passing an empty string");
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "age", "Nitish"));
+    QVERIFY(utils.testDisconnectAndDropDb());
+}
+
+static QRegularExpression resultRegExp(const QString &code, const QString &message,
+                                       const QString &sql, const QString &serverErrorCode,
+                                       const QString &serverMessage)
+{
+    return QRegularExpression(
+        QString::fromLatin1("KDbResult: CODE=%1 MESSAGE=\\\"%2\\\" ERR_SQL=KDbEscapedString:"
+                            "\\\"%3\\\"  SERVER_ERROR_CODE=%4 SERVER_MESSAGE=\\\"%5")
+                .arg(code.isEmpty() ? "[0-9]*" : code, message, sql,
+                     serverErrorCode.isEmpty() ? "[0-9]*" : serverErrorCode, serverMessage));
+}
+
+void KDbTest::deleteNonExistingRecordTest()
+{
+    QVERIFY(utils.testCreateDbWithTables("KDbTest"));
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "id", 400));
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "name", "FooBar"));
+    QTest::ignoreMessage(QtWarningMsg, resultRegExp("260", "Error while executing SQL statement.",
+        "DELETE FROM \\[persons\\] WHERE \\[Foo\\]='FooBar'", "0", "no such column: Foo"));
+    QVERIFY2(!KDb::deleteRecords(utils.connection.data(), "persons", "Foo", "FooBar"),
+             "Passing a NonExisting Column - should fail because 'Foo' column does not exist, "
+             "See also https://bugs.kde.org/376052");
+    QVERIFY(utils.testDisconnectAndDropDb());
+}
+
+void KDbTest::deleteRecordWithTwoConstraintsTest()
+{
+    QVERIFY(utils.testCreateDbWithTables("KDbTest"));
+    QVERIFY2(KDb::deleteRecords(utils.connection.data(), "persons", "id", KDbField::Integer,
+                                2, "age", KDbField::Integer, 60),
+             "Both fields are INTEGER");
+    KDbTableSchema *kdb_t = utils.connection.data()->tableSchema("persons");
+    QVERIFY(kdb_t);
+    utils.connection.data()->insertRecord(kdb_t, 10, QVariant(), "Foo", "Bar") ;
+    QVERIFY2(KDb::deleteRecords(utils.connection.data(), "persons", "id", KDbField::Integer,
+                                10, "age", KDbField::Integer, QVariant()),
+             "Passing NULL value for integer field");
+    QVERIFY(utils.connection.data()->insertRecord(kdb_t, 20, QVariant(), QVariant(), "Bar"));
+    QVERIFY2(KDb::deleteRecords(utils.connection.data(), "persons", "age", KDbField::Integer,
+                                QVariant(), "name", KDbField::Text, QVariant()),
+             "Passing 2 NULL values");
+    QVERIFY2(KDb::deleteRecords(utils.connection.data(), "persons", "age", KDbField::Integer,
+                                20, "name", KDbField::Text, "Jaroslaw"),
+             "One argument is Integer and another is Text");
+    QVERIFY2(KDb::deleteRecords(utils.connection.data(), "persons", "age", KDbField::Integer,
+                                20, "name", KDbField::Text, 56),
+             "Two arguments, passing second integer instead of text but it is converted to text");
+    QTest::ignoreMessage(QtWarningMsg, resultRegExp("260", "Error while executing SQL statement.",
+        "DELETE FROM \\[persons\\] WHERE \\[age\\]=TRAP AND \\[name\\]='56'", "0", "no such column: TRAP"));
+    QVERIFY2(!KDb::deleteRecords(utils.connection.data(), "persons", "age", KDbField::Integer,
+                                 "TRAP", "name", KDbField::Text, 56),
+             "Passing text instead of integer, conversion error expected");
+    QVERIFY(utils.testDisconnectAndDropDb());
+}
+
+void  KDbTest::deleteRecordWithThreeConstraintsTest()
+{
+    QVERIFY(utils.testCreateDbWithTables("KDbTest"));
+    KDbTableSchema *kdb_t = utils.connection.data()->tableSchema("persons");
+    QVERIFY(kdb_t);
+    //One null value.
+    QVERIFY(utils.connection.data()->insertRecord(kdb_t, 10, QVariant(), "Foo", "Bar"));
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "age", KDbField::Integer, QVariant(),
+                                "name", KDbField::Text, "Foo", "surname", KDbField::Text, "Bar"));
+    //Mix of null and empty values
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "age", KDbField::Integer, QVariant(),
+                                "name", KDbField::Text, "", "surname", KDbField::Text, ""));
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "age", KDbField::Integer,27,
+                                "name", KDbField::Text, "Jaraslaw", "id", KDbField::Integer, 1));
+    QVERIFY(KDb::deleteRecords(utils.connection.data(), "persons", "age", KDbField::Integer, 60,
+                                "name", KDbField::Text, "Lech", "id", KDbField::Integer, 2));
+    QVERIFY(utils.testDisconnectAndDropDb());
+}
+
+void KDbTest::deleteAllRecordsTest()
+{
+    QVERIFY(utils.testCreateDbWithTables("KDbTest"));
+    QVERIFY(KDb::deleteAllRecords(utils.connection.data(), "persons"));
+
+    QRegularExpression deleteAllErrorRegExp = resultRegExp(
+        "", "Error while executing SQL statement.", "DELETE FROM \\[.*\\]", 0, "no such table: .*");
+    QTest::ignoreMessage(QtWarningMsg, deleteAllErrorRegExp);
+    QVERIFY2(!KDb::deleteAllRecords(utils.connection.data(), QString()),
+             "Passing a null table name");
+    QTest::ignoreMessage(QtWarningMsg, deleteAllErrorRegExp);
+    QVERIFY2(!KDb::deleteAllRecords(utils.connection.data(), ""),
+             "Passing an empty table name");
+    QVERIFY(KDb::deleteAllRecords(utils.connection.data(), "cars"));
+    QTest::ignoreMessage(QtWarningMsg, deleteAllErrorRegExp);
+    QVERIFY2(!KDb::deleteAllRecords(utils.connection.data(), "NonExistingTable"),
+             "Passing a nonexisting table name");
+    QVERIFY(utils.testDisconnectAndDropDb());
+}
+
 void KDbTest::cleanupTestCase()
 {
 }
