@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2004-2016 Jarosław Staniek <staniek@kde.org>
+   Copyright (C) 2004-2017 Jarosław Staniek <staniek@kde.org>
    Copyright (c) 2006, 2007 Thomas Braxton <kde.braxton@gmail.com>
    Copyright (c) 1999 Preston Brown <pbrown@kde.org>
    Copyright (c) 1997 Matthias Kalle Dalheimer <kalle@kde.org>
@@ -21,21 +21,22 @@
 */
 
 #include "KDb.h"
-#include "KDb_p.h"
 #include "KDbConnectionData.h"
 #include "KDbConnection.h"
 #include "KDbCursor.h"
-#include "KDbDriverManager.h"
+#include "kdb_debug.h"
 #include "KDbDriverBehavior.h"
+#include "KDbDriverManager.h"
+#include "KDbDriver_p.h"
 #include "KDbLookupFieldSchema.h"
 #include "KDbMessageHandler.h"
 #include "KDbNativeStatementBuilder.h"
+#include "KDb_p.h"
 #include "KDbQuerySchema.h"
 #include "KDbRecordData.h"
 #include "KDbSqlResult.h"
 #include "KDbTableOrQuerySchema.h"
 #include "KDbVersionInfo.h"
-#include "kdb_debug.h"
 #include "transliteration/transliteration_table.h"
 
 #include <QMap>
@@ -64,7 +65,7 @@ class ConnectionTestThread : public QThread
     Q_OBJECT
 public:
     ConnectionTestThread(ConnectionTestDialog *dlg, const KDbConnectionData& connData);
-    virtual void run();
+    void run() override;
 Q_SIGNALS:
     void error(const QString& msg, const QString& details);
 protected:
@@ -83,18 +84,21 @@ class ConnectionTestDialog : public QProgressDialog // krazy:exclude=qclasses
 public:
     ConnectionTestDialog(const KDbConnectionData& data, KDbMessageHandler* msgHandler,
                          QWidget* parent = nullptr);
-    virtual ~ConnectionTestDialog();
+    ~ConnectionTestDialog() override;
 
-    int exec();
+    int exec() override;
 
 public Q_SLOTS:
     void error(const QString& msg, const QString& details);
 
 protected Q_SLOTS:
     void slotTimeout();
-    virtual void reject();
+    void accept() override;
+    void reject() override;
 
 protected:
+    void finish();
+
     QPointer<ConnectionTestThread> m_thread;
     KDbConnectionData m_connData;
     QTimer m_timer;
@@ -120,7 +124,7 @@ ConnectionTestThread::ConnectionTestThread(ConnectionTestDialog* dlg, const KDbC
     m_driver = manager.driver(m_connData.driverId());
     if (manager.result().isError()) {
         emitError(*manager.resultable());
-        m_driver = 0;
+        m_driver = nullptr;
     }
 }
 
@@ -203,22 +207,22 @@ int ConnectionTestDialog::exec()
 void ConnectionTestDialog::slotTimeout()
 {
     //kdbDebug() << "tid:" << QThread::currentThread() << "this_thread:" << thread();
-    kdbDebug() << m_error;
+    //kdbDebug() << m_error;
     bool notResponding = false;
     if (m_elapsedTime >= 1000*5) {//5 seconds
         m_stopWaiting = true;
         notResponding = true;
     }
-    kdbDebug() << m_elapsedTime << m_stopWaiting << notResponding;
+    //kdbDebug() << m_elapsedTime << m_stopWaiting << notResponding;
     if (m_stopWaiting) {
         m_timer.disconnect(this);
         m_timer.stop();
-        reject();
-        kdbDebug() << "after reject";
         QString message;
         QString details;
         KDbMessageHandler::MessageType type;
         if (m_error) {
+            reject();
+            //kdbDebug() << "after reject";
             message = tr("Test connection to \"%1\" database server failed.")
                          .arg(m_connData.toUserVisibleString());
             details = m_msg;
@@ -228,10 +232,14 @@ void ConnectionTestDialog::slotTimeout()
             type = KDbMessageHandler::Sorry;
             m_error = false;
         } else if (notResponding) {
+            reject();
+            //kdbDebug() << "after reject";
             message = tr("Test connection to \"%1\" database server failed. The server is not responding.")
                          .arg(m_connData.toUserVisibleString());
             type = KDbMessageHandler::Sorry;
         } else {
+            accept();
+            //kdbDebug() << "after accept";
             message = tr("Test connection to \"%1\" database server established successfully.")
                          .arg(m_connData.toUserVisibleString()),
             type = KDbMessageHandler::Information;
@@ -248,24 +256,35 @@ void ConnectionTestDialog::slotTimeout()
 void ConnectionTestDialog::error(const QString& msg, const QString& details)
 {
     //kdbDebug() << "tid:" << QThread::currentThread() << "this_thread:" << thread();
-    kdbDebug() << msg << details;
+    //kdbDebug() << msg << details;
     m_stopWaiting = true;
     m_msg = msg;
     m_details = details;
     m_error = !msg.isEmpty() || !details.isEmpty();
     if (m_error) {
-        kdbDebug() << "ERR!";
+        kdbDebug() << "Error:" << msg << details;
     }
 }
 
+void ConnectionTestDialog::accept()
+{
+    finish();
+    QProgressDialog::accept(); // krazy:exclude=qclasses
+}
+
 void ConnectionTestDialog::reject()
+{
+    finish();
+    QProgressDialog::reject(); // krazy:exclude=qclasses
+}
+
+void ConnectionTestDialog::finish()
 {
     if (m_thread->isRunning()) {
         m_thread->terminate();
     }
     m_timer.disconnect(this);
     m_timer.stop();
-    QProgressDialog::reject(); // krazy:exclude=qclasses
 }
 
 // ----
@@ -322,24 +341,26 @@ KDbVersionInfo KDb::version()
 bool KDb::deleteRecords(KDbConnection* conn, const QString &tableName,
                         const QString &keyname, KDbField::Type keytype, const QVariant &keyval)
 {
-    Q_ASSERT(conn);
-    return conn->executeVoidSQL(KDbEscapedString("DELETE FROM %1 WHERE %2=%3")
-                            .arg(conn->escapeIdentifier(tableName))
-                            .arg(conn->escapeIdentifier(keyname))
-                            .arg(conn->driver()->valueToSQL(keytype, keyval)));
+    return conn
+        ? conn->executeSql(KDbEscapedString("DELETE FROM %1 WHERE %2=%3")
+                               .arg(conn->escapeIdentifier(tableName))
+                               .arg(conn->escapeIdentifier(keyname))
+                               .arg(conn->driver()->valueToSql(keytype, keyval)))
+        : false;
 }
 
 bool KDb::deleteRecords(KDbConnection* conn, const QString &tableName,
                         const QString &keyname1, KDbField::Type keytype1, const QVariant& keyval1,
                         const QString &keyname2, KDbField::Type keytype2, const QVariant& keyval2)
 {
-    Q_ASSERT(conn);
-    return conn->executeVoidSQL(KDbEscapedString("DELETE FROM %1 WHERE %2=%3 AND %4=%5")
-                            .arg(conn->escapeIdentifier(tableName))
-                            .arg(conn->escapeIdentifier(keyname1))
-                            .arg(conn->driver()->valueToSQL(keytype1, keyval1))
-                            .arg(conn->escapeIdentifier(keyname2))
-                            .arg(conn->driver()->valueToSQL(keytype2, keyval2)));
+    return conn
+        ? conn->executeSql(KDbEscapedString("DELETE FROM %1 WHERE %2=%3 AND %4=%5")
+                               .arg(conn->escapeIdentifier(tableName))
+                               .arg(conn->escapeIdentifier(keyname1))
+                               .arg(conn->driver()->valueToSql(keytype1, keyval1))
+                               .arg(conn->escapeIdentifier(keyname2))
+                               .arg(conn->driver()->valueToSql(keytype2, keyval2)))
+        : false;
 }
 
 bool KDb::deleteRecords(KDbConnection* conn, const QString &tableName,
@@ -347,28 +368,33 @@ bool KDb::deleteRecords(KDbConnection* conn, const QString &tableName,
                         const QString &keyname2, KDbField::Type keytype2, const QVariant& keyval2,
                         const QString &keyname3, KDbField::Type keytype3, const QVariant& keyval3)
 {
-    Q_ASSERT(conn);
-    return conn->executeVoidSQL(KDbEscapedString("DELETE FROM %1 WHERE %2=%3 AND %4=%5 AND %6=%7")
-                            .arg(conn->escapeIdentifier(tableName))
-                            .arg(conn->escapeIdentifier(keyname1))
-                            .arg(conn->driver()->valueToSQL(keytype1, keyval1))
-                            .arg(conn->escapeIdentifier(keyname2))
-                            .arg(conn->driver()->valueToSQL(keytype2, keyval2))
-                            .arg(conn->escapeIdentifier(keyname3))
-                            .arg(conn->driver()->valueToSQL(keytype3, keyval3)));
+    return conn
+        ? conn->executeSql(KDbEscapedString("DELETE FROM %1 WHERE %2=%3 AND %4=%5 AND %6=%7")
+                               .arg(conn->escapeIdentifier(tableName))
+                               .arg(conn->escapeIdentifier(keyname1))
+                               .arg(conn->driver()->valueToSql(keytype1, keyval1))
+                               .arg(conn->escapeIdentifier(keyname2))
+                               .arg(conn->driver()->valueToSql(keytype2, keyval2))
+                               .arg(conn->escapeIdentifier(keyname3))
+                               .arg(conn->driver()->valueToSql(keytype3, keyval3)))
+        : false;
 }
 
 bool KDb::deleteAllRecords(KDbConnection* conn, const QString &tableName)
 {
-    Q_ASSERT(conn);
-    return conn->executeVoidSQL(KDbEscapedString("DELETE FROM %1")
-                            .arg(conn->escapeIdentifier(tableName)));
+    return conn
+        ? conn->executeSql(
+              KDbEscapedString("DELETE FROM %1").arg(conn->escapeIdentifier(tableName)))
+        : false;
 }
 
-KDB_EXPORT quint64 KDb::lastInsertedAutoIncValue(KDbSqlResult *result,
-    const QString& autoIncrementFieldName, const QString& tableName, quint64* recordId)
+KDB_EXPORT quint64 KDb::lastInsertedAutoIncValue(QSharedPointer<KDbSqlResult> result,
+                                                 const QString &autoIncrementFieldName,
+                                                 const QString &tableName, quint64 *recordId)
 {
-    Q_ASSERT(result);
+    if (!result) {
+        return std::numeric_limits<quint64>::max();
+    }
     const quint64 foundRecordId = result->lastInsertRecordId();
     if (recordId) {
         *recordId = foundRecordId;
@@ -378,9 +404,10 @@ KDB_EXPORT quint64 KDb::lastInsertedAutoIncValue(KDbSqlResult *result,
 }
 
 KDB_EXPORT quint64 KDb::lastInsertedAutoIncValue(KDbConnection *conn, const quint64 recordId,
-    const QString& autoIncrementFieldName, const QString& tableName)
+                                                 const QString &autoIncrementFieldName,
+                                                 const QString &tableName)
 {
-    const KDbDriverBehavior *behavior = KDbDriverBehavior::get(conn->driver());
+    const KDbDriverBehavior *behavior = KDbDriverPrivate::behavior(conn->driver());
     if (behavior->ROW_ID_FIELD_RETURNS_LAST_AUTOINCREMENTED_VALUE) {
         return recordId;
     }
@@ -413,8 +440,8 @@ KDbEscapedString KDb::sqlWhere(KDbDriver *drv, KDbField::Type t,
                             const QString& fieldName, const QVariant& value)
 {
     if (value.isNull())
-        return KDbEscapedString(fieldName) + " is NULL";
-    return KDbEscapedString(fieldName) + '=' + drv->valueToSQL(t, value);
+        return KDbEscapedString(fieldName) + " IS NULL";
+    return KDbEscapedString(fieldName) + '=' + drv->valueToSql(t, value);
 }
 
 //! Cache
@@ -476,8 +503,14 @@ KDbField::Type KDb::defaultFieldTypeForGroup(KDbField::TypeGroup typeGroup)
 
 void KDb::getHTMLErrorMesage(const KDbResultable& resultable, QString *msg, QString *details)
 {
-    Q_ASSERT(msg);
-    Q_ASSERT(details);
+    if (!msg) {
+        kdbWarning() << "Missing 'msg' parameter";
+        return;
+    }
+    if (!details) {
+        kdbWarning() << "Missing 'details' parameter";
+        return;
+    }
     const KDbResult result(resultable.result());
     if (!result.isError())
         return;
@@ -493,9 +526,9 @@ void KDb::getHTMLErrorMesage(const KDbResultable& resultable, QString *msg, QStr
     if (!result.serverMessage().isEmpty())
         *details += QLatin1String("<p><b>") + kdb::tr("Message from server:")
                    + QLatin1String("</b> ") + result.serverMessage();
-    if (!result.recentSQLString().isEmpty())
+    if (!result.recentSqlString().isEmpty())
         *details += QLatin1String("<p><b>") + kdb::tr("SQL statement:")
-                   + QString::fromLatin1("</b> <tt>%1</tt>").arg(result.recentSQLString().toString());
+                   + QString::fromLatin1("</b> <tt>%1</tt>").arg(result.recentSqlString().toString());
     int serverErrorCode = 0;
     QString serverResultName;
     if (result.isError()) {
@@ -504,7 +537,7 @@ void KDb::getHTMLErrorMesage(const KDbResultable& resultable, QString *msg, QStr
     }
     if (   !details->isEmpty()
         && (   !result.serverMessage().isEmpty()
-            || !result.recentSQLString().isEmpty()
+            || !result.recentSqlString().isEmpty()
             || !serverResultName.isEmpty()
             || serverErrorCode != 0)
            )
@@ -530,101 +563,49 @@ void KDb::getHTMLErrorMesage(const KDbResultable& resultable, QString *msg, QStr
 
 void KDb::getHTMLErrorMesage(const KDbResultable& resultable, QString *msg)
 {
-    Q_ASSERT(msg);
     getHTMLErrorMesage(resultable, msg, msg);
 }
 
 void KDb::getHTMLErrorMesage(const KDbResultable& resultable, KDbResultInfo *info)
 {
-    Q_ASSERT(info);
-    getHTMLErrorMesage(resultable, &info->msg, &info->desc);
+    if (!info) {
+        kdbWarning() << "Missing 'info' parameter";
+        return;
+    }
+    getHTMLErrorMesage(resultable, &info->message, &info->description);
 }
 
 tristate KDb::idForObjectName(KDbConnection* conn, int *id, const QString& objName, int objType)
 {
-    Q_ASSERT(conn);
-    Q_ASSERT(id);
-    return conn->querySingleNumber(
-        KDbEscapedString("SELECT o_id FROM kexi__objects WHERE o_name=%1 AND o_type=%2")
-        .arg(conn->escapeString(objName)).arg(objType), id);
+    return conn
+        ? conn->querySingleNumber(
+              KDbEscapedString("SELECT o_id FROM kexi__objects WHERE o_name=%1 AND o_type=%2")
+                  .arg(conn->escapeString(objName))
+                  .arg(objType),
+              id)
+        : false;
 }
 
 //-----------------------------------------
 
-void KDb::connectionTestDialog(QWidget* parent, const KDbConnectionData& data,
-                               KDbMessageHandler* msgHandler)
+tristate KDb::showConnectionTestDialog(QWidget *parent, const KDbConnectionData &data,
+                                   KDbMessageHandler *msgHandler)
 {
     ConnectionTestDialog dlg(data, msgHandler, parent);
-    dlg.exec();
-}
-
-int KDb::recordCount(KDbConnection* conn, const KDbEscapedString& sql)
-{
-    int count = -1; //will be changed only on success of querySingleNumber()
-    conn->querySingleNumber(KDbEscapedString("SELECT COUNT() FROM (") + sql
-        + ") AS kdb__subquery", &count);
-    return count;
-}
-
-int KDb::recordCount(const KDbTableSchema& tableSchema)
-{
-//! @todo does not work with non-SQL data sources
-    if (!tableSchema.connection()) {
-        kdbWarning() << "no tableSchema.connection()";
-        return -1;
+    const int result = dlg.exec();
+    if (dlg.wasCanceled()) {
+        return cancelled;
     }
-    int count = -1; //will be changed only on success of querySingleNumber()
-    tableSchema.connection()->querySingleNumber(
-        KDbEscapedString("SELECT COUNT(*) FROM ")
-        + tableSchema.connection()->escapeIdentifier(tableSchema.name()),
-        &count
-    );
-    return count;
-}
-
-int KDb::recordCount(KDbQuerySchema* querySchema, const QList<QVariant>& params)
-{
-//! @todo does not work with non-SQL data sources
-    if (!querySchema->connection()) {
-        kdbWarning() << "no querySchema->connection()";
-        return -1;
-    }
-    int count = -1; //will be changed only on success of querySingleNumber()
-    KDbNativeStatementBuilder builder(querySchema->connection());
-    KDbEscapedString subSql;
-    if (!builder.generateSelectStatement(&subSql, querySchema, params)) {
-        return -1;
-    }
-    tristate result = querySchema->connection()->querySingleNumber(
-        KDbEscapedString("SELECT COUNT(*) FROM (") + subSql + ") AS kdb__subquery", &count
-    );
-    return true == result ? count : -1;
-}
-
-int KDb::recordCount(KDbTableOrQuerySchema* tableOrQuery, const QList<QVariant>& params)
-{
-    if (tableOrQuery->table())
-        return recordCount(*tableOrQuery->table());
-    if (tableOrQuery->query())
-        return recordCount(tableOrQuery->query(), params);
-    return -1;
-}
-
-int KDb::fieldCount(KDbTableOrQuerySchema* tableOrQuery)
-{
-    if (tableOrQuery->table())
-        return tableOrQuery->table()->fieldCount();
-    if (tableOrQuery->query())
-        return tableOrQuery->query()->fieldsExpanded().count();
-    return -1;
+    return result == QDialog::Accepted;
 }
 
 bool KDb::splitToTableAndFieldParts(const QString& string,
                                           QString *tableName, QString *fieldName,
                                           SplitToTableAndFieldPartsOptions option)
 {
-    Q_ASSERT(tableName);
-    Q_ASSERT(fieldName);
+    if (!tableName || !fieldName) {
+        return false;
+    }
     const int id = string.indexOf(QLatin1Char('.'));
     if (option & SetFieldNameIfNoTableName && id == -1) {
         tableName->clear();
@@ -674,19 +655,20 @@ QString KDb::numberToString(double value, int decimalPlaces)
     return ::numberToString(value, decimalPlaces, nullptr);
 }
 
-QString KDb::numberToLocaleString(double value, int decimalPlaces, const QLocale *locale)
+QString KDb::numberToLocaleString(double value, int decimalPlaces)
 {
-    if (locale) {
-        return ::numberToString(value, decimalPlaces, locale);
-    }
     QLocale defaultLocale;
     return ::numberToString(value, decimalPlaces, &defaultLocale);
 }
 
+QString KDb::numberToLocaleString(double value, int decimalPlaces, const QLocale &locale)
+{
+    return ::numberToString(value, decimalPlaces, &locale);
+}
+
 KDbField::Type KDb::intToFieldType(int type)
 {
     if (type < int(KDbField::InvalidType) || type > int(KDbField::LastType)) {
-        kdbWarning() << "invalid type" << type;
         return KDbField::InvalidType;
     }
     return static_cast<KDbField::Type>(type);
@@ -695,7 +677,6 @@ KDbField::Type KDb::intToFieldType(int type)
 KDbField::TypeGroup KDb::intToFieldTypeGroup(int typeGroup)
 {
     if (typeGroup < int(KDbField::InvalidGroup) || typeGroup > int(KDbField::LastTypeGroup)) {
-        kdbWarning() << "invalid type group" << typeGroup;
         return KDbField::InvalidGroup;
     }
     return static_cast<KDbField::TypeGroup>(typeGroup);
@@ -706,8 +687,12 @@ static bool setIntToFieldType(KDbField *field, const QVariant& value)
     Q_ASSERT(field);
     bool ok;
     const int intType = value.toInt(&ok);
-    if (!ok || KDbField::InvalidType == KDb::intToFieldType(intType)) {//for sanity
-        kdbWarning() << "invalid type";
+    if (!ok) {//for sanity
+        kdbWarning() << "Could not convert value" << value << "to field type";
+        return false;
+    }
+    if (KDbField::InvalidType == KDb::intToFieldType(intType)) {//for sanity
+        kdbWarning() << "Invalid field type" << intType;
         return false;
     }
     field->setType((KDbField::Type)intType);
@@ -771,8 +756,10 @@ static QVariant visibleColumnValue(const KDbLookupFieldSchema *lookup)
 
 void KDb::getProperties(const KDbLookupFieldSchema *lookup, QMap<QByteArray, QVariant> *values)
 {
-    Q_ASSERT(values);
-    KDbLookupFieldSchema::RecordSource recordSource;
+    if (!values) {
+        return;
+    }
+    KDbLookupFieldSchemaRecordSource recordSource;
     if (lookup) {
         recordSource = lookup->recordSource();
     }
@@ -798,7 +785,9 @@ void KDb::getProperties(const KDbLookupFieldSchema *lookup, QMap<QByteArray, QVa
 
 void KDb::getFieldProperties(const KDbField &field, QMap<QByteArray, QVariant> *values)
 {
-    Q_ASSERT(values);
+    if (!values) {
+        return;
+    }
     values->clear();
     // normal values
     values->insert("type", field.type());
@@ -824,7 +813,7 @@ void KDb::getFieldProperties(const KDbField &field, QMap<QByteArray, QVariant> *
         values->insert("visibleDecimalPlaces", field.defaultValue());
     }
     // insert lookup-related values
-    KDbLookupFieldSchema *lookup = field.table()->lookupFieldSchema(field);
+    const KDbLookupFieldSchema *lookup = field.table()->lookupFieldSchema(field);
     KDb::getProperties(lookup, values);
 }
 
@@ -842,7 +831,9 @@ static bool containsLookupFieldSchemaProperties(const QMap<QByteArray, QVariant>
 
 bool KDb::setFieldProperties(KDbField *field, const QMap<QByteArray, QVariant>& values)
 {
-    Q_ASSERT(field);
+    if (!field) {
+        return false;
+    }
     QMap<QByteArray, QVariant>::ConstIterator it;
     if ((it = values.find("type")) != values.constEnd()) {
         if (!setIntToFieldType(field, *it))
@@ -926,7 +917,7 @@ bool KDb::setFieldProperties(KDbField *field, const QMap<QByteArray, QVariant>&
             if (createdLookup) {
                 if (field->table()->setLookupFieldSchema(field->name(), lookup)) {
                     createdLookup.take(); // ownership passed
-                    lookup = 0;
+                    lookup = nullptr;
                 }
             }
         }
@@ -968,7 +959,7 @@ bool KDb::isExtendedTableFieldProperty(const QByteArray& propertyName)
 struct KDb_LookupFieldSchemaProperties {
     KDb_LookupFieldSchemaProperties() {
         QMap<QByteArray, QVariant> tmp;
-        KDb::getProperties(0, &tmp);
+        KDb::getProperties(nullptr, &tmp);
         for (QMap<QByteArray, QVariant>::ConstIterator it(tmp.constBegin());
              it != tmp.constEnd(); ++it)
         {
@@ -988,7 +979,9 @@ bool KDb::isLookupFieldSchemaProperty(const QByteArray& propertyName)
 
 bool KDb::setFieldProperty(KDbField *field, const QByteArray& propertyName, const QVariant& value)
 {
-    Q_ASSERT(field);
+    if (!field) {
+        return false;
+    }
 #define SET_BOOLEAN_FLAG(flag, value) { \
         constraints |= KDbField::flag; \
         if (!value) \
@@ -1094,7 +1087,7 @@ bool KDb::setFieldProperty(KDbField *field, const QByteArray& propertyName, cons
         field->setCustomProperty(propertyName, value);
     }
 
-    kdbWarning() << "property" << propertyName << "not found!";
+    kdbWarning() << "Field property" << propertyName << "not found!";
     return false;
 #undef SET_BOOLEAN_FLAG
 #undef GET_INT
@@ -1165,7 +1158,7 @@ QVariant KDb::loadPropertyValueFromDom(const QDomNode& node, bool* ok)
     }
     else {
 //! @todo add more QVariant types
-        kdbWarning() << "KDb::loadPropertyValueFromDom(): unknown type '" << valueType << "'";
+        kdbWarning() << "Unknown property type" << valueType;
     }
     if (ok)
         *ok = false;
@@ -1175,8 +1168,9 @@ QVariant KDb::loadPropertyValueFromDom(const QDomNode& node, bool* ok)
 QDomElement KDb::saveNumberElementToDom(QDomDocument *doc, QDomElement *parentEl,
         const QString& elementName, int value)
 {
-    Q_ASSERT(doc);
-    Q_ASSERT(parentEl);
+    if (!doc || !parentEl || elementName.isEmpty()) {
+        return QDomElement();
+    }
     QDomElement el(doc->createElement(elementName));
     parentEl->appendChild(el);
     QDomElement numberEl(doc->createElement(QLatin1String("number")));
@@ -1188,8 +1182,9 @@ QDomElement KDb::saveNumberElementToDom(QDomDocument *doc, QDomElement *parentEl
 QDomElement KDb::saveBooleanElementToDom(QDomDocument *doc, QDomElement *parentEl,
         const QString& elementName, bool value)
 {
-    Q_ASSERT(doc);
-    Q_ASSERT(parentEl);
+    if (!doc || !parentEl || elementName.isEmpty()) {
+        return QDomElement();
+    }
     QDomElement el(doc->createElement(elementName));
     parentEl->appendChild(el);
     QDomElement numberEl(doc->createElement(QLatin1String("bool")));
@@ -1237,7 +1232,7 @@ QVariant KDb::emptyValueForFieldType(KDbField::Type type)
         if (type == KDbField::Time)
             return QTime::currentTime();
     }
-    kdbWarning() << "no value for type" << KDbField::typeName(type);
+    kdbWarning() << "No empty value for field type" << KDbField::typeName(type);
     return QVariant();
 }
 
@@ -1290,7 +1285,7 @@ QVariant KDb::notEmptyValueForFieldType(KDbField::Type type)
         if (type == KDbField::Time)
             return QTime::currentTime();
     }
-    kdbWarning() << "no value for type" << KDbField::typeName(type);
+    kdbWarning() << "No non-empty value for field type" << KDbField::typeName(type);
     return QVariant();
 }
 
@@ -1379,6 +1374,16 @@ QString KDb::escapeString(const QString& string)
     return newString;
 }
 
+KDbEscapedString KDb::escapeString(KDbDriver *drv, const QString& string)
+{
+    return drv ? drv->escapeString(string) : KDbEscapedString(KDb::escapeString(string));
+}
+
+KDbEscapedString KDb::escapeString(KDbConnection *conn, const QString& string)
+{
+    return conn ? conn->escapeString(string) : KDbEscapedString(KDb::escapeString(string));
+}
+
 //! @see handleHex()
 const int CODE_POINT_DIGITS = std::numeric_limits<int>::max();
 //! @see handleHex()
@@ -1589,33 +1594,33 @@ inline static char intToHexDigit(unsigned char val)
 QString KDb::escapeBLOB(const QByteArray& array, BLOBEscapingType type)
 {
     const int size = array.size();
-    if (size == 0 && type == BLOBEscape0xHex)
+    if (size == 0 && type == BLOBEscapingType::ZeroXHex)
         return QString();
     int escaped_length = size * 2;
-    if (type == BLOBEscape0xHex || type == BLOBEscapeOctal)
+    if (type == BLOBEscapingType::ZeroXHex || type == BLOBEscapingType::Octal)
         escaped_length += 2/*0x or X'*/;
-    else if (type == BLOBEscapeXHex)
+    else if (type == BLOBEscapingType::XHex)
         escaped_length += 3; //X' + '
-    else if (type == BLOBEscapeByteaHex)
+    else if (type == BLOBEscapingType::ByteaHex)
         escaped_length += (4 + 8); // E'\x + '::bytea
 
     QString str;
     str.reserve(escaped_length);
     if (str.capacity() < escaped_length) {
-        kdbWarning() << "no enough memory (cannot allocate" << escaped_length << "chars)";
+        kdbWarning() << "Not enough memory (cannot allocate" << escaped_length << "characters)";
         return QString();
     }
-    if (type == BLOBEscapeXHex)
+    if (type == BLOBEscapingType::XHex)
         str = QString::fromLatin1("X'");
-    else if (type == BLOBEscape0xHex)
+    else if (type == BLOBEscapingType::ZeroXHex)
         str = QString::fromLatin1("0x");
-    else if (type == BLOBEscapeOctal)
+    else if (type == BLOBEscapingType::Octal)
         str = QString::fromLatin1("'");
-    else if (type == BLOBEscapeByteaHex)
+    else if (type == BLOBEscapingType::ByteaHex)
         str = QString::fromLatin1("E'\\\\x");
 
     int new_length = str.length(); //after X' or 0x, etc.
-    if (type == BLOBEscapeOctal) {
+    if (type == BLOBEscapingType::Octal) {
         // only escape nonprintable characters as in Table 8-7:
         // http://www.postgresql.org/docs/8.1/interactive/datatype-binary.html
         // i.e. escape for bytes: < 32, >= 127, 39 ('), 92(\).
@@ -1638,9 +1643,9 @@ QString KDb::escapeBLOB(const QByteArray& array, BLOBEscapingType type)
             str[new_length++] = intToHexDigit(val % 16);
         }
     }
-    if (type == BLOBEscapeXHex || type == BLOBEscapeOctal) {
+    if (type == BLOBEscapingType::XHex || type == BLOBEscapingType::Octal) {
         str[new_length++] = '\'';
-    } else if (type == BLOBEscapeByteaHex) {
+    } else if (type == BLOBEscapingType::ByteaHex) {
         str[new_length++] = '\'';
         str[new_length++] = ':';
         str[new_length++] = ':';
@@ -1689,7 +1694,7 @@ QByteArray KDb::pgsqlByteaToByteArray(const char* data, int length)
                         array[output] = char((int(s[1] - '0') * 8 + int(s[2] - '0')) * 8 + int(s[3] - '0'));
                     s += 4;
                 } else {
-                    kdbWarning() << "no octal value after backslash";
+                    kdbWarning() << "Missing octal value after backslash";
                     s++;
                 }
             } else {
@@ -1731,7 +1736,7 @@ QByteArray KDb::xHexToByteArray(const char* data, int length, bool *ok)
 
 /*! \return byte array converted from \a data of length \a length.
  \a data is escaped in format 0x*, where * is one or more bytes in hexadecimal format.
- See BLOBEscape0xHex. */
+ See BLOBEscapingType::ZeroXHex. */
 QByteArray KDb::zeroXHexToByteArray(const char* data, int length, bool *ok)
 {
     if (length < 0) {
@@ -1843,7 +1848,7 @@ QList<int> KDb::deserializeIntList(const QString &data, bool *ok)
 QString KDb::variantToString(const QVariant& v)
  {
     if (v.type() == QVariant::ByteArray) {
-        return KDb::escapeBLOB(v.toByteArray(), KDb::BLOBEscapeHex);
+        return KDb::escapeBLOB(v.toByteArray(), KDb::BLOBEscapingType::Hex);
     }
     else if (v.type() == QVariant::StringList) {
         return serializeList(v.toStringList());
@@ -1907,8 +1912,9 @@ bool KDb::isDefaultValueAllowed(const KDbField &field)
 void KDb::getLimitsForFieldType(KDbField::Type type, qlonglong *minValue, qlonglong *maxValue,
                                 Signedness signedness)
 {
-    Q_ASSERT(minValue);
-    Q_ASSERT(maxValue);
+    if (!minValue || !maxValue) {
+        return;
+    }
     switch (type) {
     case KDbField::Byte:
 //! @todo always ok?
@@ -2048,7 +2054,6 @@ QStringList KDb::libraryPaths()
 
 QString KDb::temporaryTableName(KDbConnection *conn, const QString &baseName)
 {
-    Q_ASSERT(conn);
     if (!conn) {
         return QString();
     }
@@ -2129,6 +2134,19 @@ bool KDb::isIdentifier(const QString& s)
     return i > 0 && i == sLength;
 }
 
+bool KDb::isIdentifier(const QByteArray& s)
+{
+    int i;
+    const int sLength = s.length();
+    for (i = 0; i < sLength; i++) {
+        const char c = s.at(i);
+        if (c == 0 || !(c == '_' || (c >= 'a' && c <= 'z') || (c >= 'A' && c <= 'Z') || (i > 0 && c >= '0' && c <= '9'))) {
+            break;
+        }
+    }
+    return i > 0 && i == sLength;
+}
+
 static inline QString charToIdentifier(const QChar& c)
 {
     if (c.unicode() >= TRANSLITERATION_TABLE_SIZE)
