@@ -27,7 +27,7 @@
 
 #include <stdlib.h>
 
-class KDbAlterTableHandler::Private
+class Q_DECL_HIDDEN KDbAlterTableHandler::Private
 {
 public:
     Private() {}
@@ -239,9 +239,6 @@ static KDbAlterTableHandler::ActionDict* createActionDict(
 static void debugAction(KDbAlterTableHandler::ActionBase *action, int nestingLevel,
                         bool simulate, const QString& prependString = QString(), QString * debugTarget = nullptr)
 {
-    Q_UNUSED(simulate);
-    Q_UNUSED(nestingLevel);
-
     QString debugString;
     if (!debugTarget)
         debugString = prependString;
@@ -264,6 +261,9 @@ static void debugAction(KDbAlterTableHandler::ActionBase *action, int nestingLev
 #ifdef KDB_DEBUG_GUI
         if (simulate)
             KDb::alterTableActionDebugGUI(debugString, nestingLevel);
+#else
+        Q_UNUSED(simulate)
+        Q_UNUSED(nestingLevel)
 #endif
     }
 }
@@ -345,8 +345,7 @@ void KDbAlterTableHandler::ChangeFieldPropertyAction::simplifyActions(ActionDict
                     adict = createActionDict( fieldActions, fieldName() );
                   adict->insert(m_propertyName.toLatin1(), newRenameAction);*/
         } else {
-            ActionBase *removeActionForThisField
-                = actionsLikeThis ? actionsLikeThis->value(":remove:") : nullptr;
+            ActionBase *removeActionForThisField = actionsLikeThis ? actionsLikeThis->value(":remove:") : nullptr;
             if (removeActionForThisField) {
                 //if this field is going to be removed, just change the action's field name
                 // and do not add a new action
@@ -372,8 +371,7 @@ void KDbAlterTableHandler::ChangeFieldPropertyAction::simplifyActions(ActionDict
         }
         return;
     }
-    ActionBase *removeActionForThisField
-        = actionsLikeThis ? actionsLikeThis->value(":remove:") : nullptr;
+    ActionBase *removeActionForThisField = actionsLikeThis ? actionsLikeThis->value(":remove:") : nullptr;
     if (removeActionForThisField) {
         //if this field is going to be removed, do not add a new action
         return;
@@ -413,7 +411,7 @@ tristate KDbAlterTableHandler::ChangeFieldPropertyAction::updateTableSchema(KDbT
         if (fieldHash->value(field->name()) == field->name())
             fieldHash->remove(field->name());
         fieldHash->insert(newValue().toString(), field->name());
-        table->renameField(field, newValue().toString());
+        (void)table->renameField(field, newValue().toString());
         return true;
     }
     return cancelled;
@@ -628,8 +626,7 @@ void KDbAlterTableHandler::InsertFieldAction::simplifyActions(ActionDictDict *fi
     // Try to find actions related to this action
     ActionDict *actionsForThisField = fieldActions->value(uid());
 
-    ActionBase *removeActionForThisField
-        = actionsForThisField ? actionsForThisField->value(":remove:") : nullptr;
+    ActionBase *removeActionForThisField = actionsForThisField ? actionsForThisField->value(":remove:") : nullptr;
     if (removeActionForThisField) {
         //if this field is going to be removed, do not add a new action
         //and remove the "Remove" action
@@ -994,7 +991,10 @@ KDbTableSchema* KDbAlterTableHandler::execute(const QString& tableName, Executio
 
     if (recreateTable) {
         // Create the destination table with temporary name
-        if (!d->conn->createTable(newTable, false)) {
+        if (!d->conn->createTable(newTable,
+          KDbConnection::CreateTableOptions(KDbConnection::CreateTableOption::Default)
+              & ~KDbConnection::CreateTableOptions(KDbConnection::CreateTableOption::DropDestination)))
+        {
             m_result = d->conn->result();
             delete newTable;
             args->result = false;
@@ -1041,30 +1041,30 @@ KDbTableSchema* KDbAlterTableHandler::execute(const QString& tableName, Executio
         KDbEscapedString sourceFields;
         foreach(KDbField* f, *newTable->fields()) {
             QString renamedFieldName(fieldHash.value(f->name()));
-            KDbEscapedString sourceSQLString;
+            KDbEscapedString sourceSqlString;
             const KDbField::Type type = f->type(); // cache: evaluating type of expressions can be expensive
             if (!renamedFieldName.isEmpty()) {
                 //this field should be renamed
-                sourceSQLString = KDbEscapedString(d->conn->escapeIdentifier(renamedFieldName));
+                sourceSqlString = KDbEscapedString(d->conn->escapeIdentifier(renamedFieldName));
             } else if (!f->defaultValue().isNull()) {
                 //this field has a default value defined
 //! @todo support expressions (eg. TODAY()) as a default value
 //! @todo this field can be notNull or notEmpty - check whether the default is ok
 //!       (or do this checking also in the Table Designer?)
-                sourceSQLString = d->conn->driver()->valueToSQL(type, f->defaultValue());
+                sourceSqlString = d->conn->driver()->valueToSql(type, f->defaultValue());
             } else if (f->isNotNull()) {
                 //this field cannot be null
-                sourceSQLString = d->conn->driver()->valueToSQL(
+                sourceSqlString = d->conn->driver()->valueToSql(
                                       type, KDb::emptyValueForFieldType(type));
             } else if (f->isNotEmpty()) {
                 //this field cannot be empty - use any nonempty value..., e.g. " " for text or 0 for number
-                sourceSQLString = d->conn->driver()->valueToSQL(
+                sourceSqlString = d->conn->driver()->valueToSql(
                                       type, KDb::notEmptyValueForFieldType(type));
             }
 //! @todo support unique, validatationRule, unsigned flags...
 //! @todo check for foreignKey values...
 
-            if (!sourceSQLString.isEmpty()) {
+            if (!sourceSqlString.isEmpty()) {
                 if (first) {
                     first = false;
                 } else {
@@ -1072,12 +1072,12 @@ KDbTableSchema* KDbAlterTableHandler::execute(const QString& tableName, Executio
                     sourceFields.append(", ");
                 }
                 sql += d->conn->escapeIdentifier(f->name());
-                sourceFields.append(sourceSQLString);
+                sourceFields.append(sourceSqlString);
             }
         }
         sql += (") SELECT " + sourceFields + " FROM " + oldTable->name());
         kdbDebug() << " ** " << sql;
-        if (!d->conn->executeVoidSQL(sql)) {
+        if (!d->conn->executeSql(sql)) {
             m_result = d->conn->result();
 //! @todo delete newTable...
             args->result = false;
@@ -1094,7 +1094,9 @@ KDbTableSchema* KDbAlterTableHandler::execute(const QString& tableName, Executio
             oldTable = 0;*/
 
         // Replace the old table with the new one (oldTable will be destroyed)
-        if (!d->conn->alterTableName(newTable, oldTableName, true /*replace*/)) {
+        if (!d->conn->alterTableName(newTable, oldTableName,
+            KDbConnection::AlterTableNameOption::Default | KDbConnection::AlterTableNameOption::DropDestination))
+        {
             m_result = d->conn->result();
 //! @todo delete newTable...
             args->result = false;
