@@ -39,8 +39,10 @@ class SqliteConnection : public KDbConnection
 public:
     ~SqliteConnection() override;
 
-    KDbCursor* prepareQuery(const KDbEscapedString& sql, int cursor_options = 0) override Q_REQUIRED_RESULT;
-    KDbCursor* prepareQuery(KDbQuerySchema* query, int cursor_options = 0) override Q_REQUIRED_RESULT;
+    KDbCursor *prepareQuery(const KDbEscapedString &sql, KDbCursor::Options options
+                            = KDbCursor::Option::None) override Q_REQUIRED_RESULT;
+    KDbCursor *prepareQuery(KDbQuerySchema *query, KDbCursor::Options options
+                            = KDbCursor::Option::None) override Q_REQUIRED_RESULT;
 
     KDbPreparedStatementInterface* prepareStatementInternal() override Q_REQUIRED_RESULT;
 
@@ -72,7 +74,7 @@ protected:
       it is defined during connection. If you pass it,
       database file name will be changed. */
     bool drv_useDatabase(const QString &dbName = QString(), bool *cancelled = nullptr,
-                                 KDbMessageHandler* msgHandler = nullptr) override;
+                         KDbMessageHandler* msgHandler = nullptr) override;
 
     bool drv_closeDatabase() override;
 
@@ -81,8 +83,9 @@ protected:
       anymore, so database file is just removed. See note from drv_useDatabase(). */
     bool drv_dropDatabase(const QString &dbName = QString()) override;
 
-    KDbSqlResult* drv_executeSQL(const KDbEscapedString& sql) override;
-    bool drv_executeVoidSQL(const KDbEscapedString& sql) override;
+    KDbSqlResult* drv_prepareSql(const KDbEscapedString& sql) override;
+
+    bool drv_executeSql(const KDbEscapedString& sql) override;
 
     //! Implemented for KDbResultable
     QString serverResultName() const override;
