@@ -38,7 +38,7 @@ public:
 
     /*! [PROTOTYPE] @return internal buffer data. */
 //! @todo virtual const char *** bufferData()
-    /*! [PROTOTYPE] @return current record data or NULL if there is no current records. */
+    /*! [PROTOTYPE] @return current record data or @c nullptr if there is no current records. */
     const char ** recordData() const override;
 
     bool drv_storeCurrentRecord(KDbRecordData* data) const override;
@@ -48,11 +48,12 @@ public:
 
 protected:
     /*! KDbCursor will operate on @a conn, raw @a sql statement will be used to execute query. */
-    SqliteCursor(SqliteConnection* conn, const KDbEscapedString& sql, int options = NoOptions);
+    SqliteCursor(SqliteConnection* conn, const KDbEscapedString& sql,
+                 Options options = KDbCursor::Option::None);
 
     /*! KDbCursor will operate on @a conn, @a query schema will be used to execute query. */
     SqliteCursor(SqliteConnection* conn, KDbQuerySchema* query,
-                 int options = NoOptions);
+                 Options options = KDbCursor::Option::None);
 
     bool drv_open(const KDbEscapedString& sql) override;
 
