@@ -37,20 +37,22 @@
 //#define KDB_ENABLE_SQLITE_SPECIFIC_FUNCTIONS
 
 //! A set of names of aggregation SQL functions
-class BuiltInAggregates : public QSet<QString>
+class BuiltInAggregates
 {
 public:
     BuiltInAggregates()
+        : data({ QStringLiteral("SUM"),
+                 QStringLiteral("MIN"),
+                 QStringLiteral("MAX"),
+                 QStringLiteral("AVG"),
+                 QStringLiteral("COUNT"),
+                 QStringLiteral("STD"),
+                 QStringLiteral("STDDEV"),
+                 QStringLiteral("VARIANCE")
+               })
     {
-        insert(QLatin1String("SUM"));
-        insert(QLatin1String("MIN"));
-        insert(QLatin1String("MAX"));
-        insert(QLatin1String("AVG"));
-        insert(QLatin1String("COUNT"));
-        insert(QLatin1String("STD"));
-        insert(QLatin1String("STDDEV"));
-        insert(QLatin1String("VARIANCE"));
     }
+    const QSet<QString> data;
 };
 
 Q_GLOBAL_STATIC(BuiltInAggregates, _builtInAggregates)
@@ -97,7 +99,9 @@ public:
             return KDbField::Null;
         }
         if (copyReturnTypeFromArg >= 0 && copyReturnTypeFromArg < argsData->children.count()) {
-            KDbQueryParameterExpressionData *queryParameterExpressionData = argsData->children.at(copyReturnTypeFromArg)->convert<KDbQueryParameterExpressionData>();
+            KDbQueryParameterExpressionData *queryParameterExpressionData
+                = argsData->children.at(copyReturnTypeFromArg)
+                      ->convert<KDbQueryParameterExpressionData>();
             if (queryParameterExpressionData) {
                 // Set query parameter type (if there are any) to deduced result type
                 //! @todo Most likely but can be also other type
@@ -128,7 +132,7 @@ class CoalesceFunctionDeclaration : public BuiltInFunctionDeclaration
 {
 public:
     CoalesceFunctionDeclaration() {}
-    virtual KDbField::Type returnType(const KDbFunctionExpressionData* f, KDbParseInfo* parseInfo) const {
+    KDbField::Type returnType(const KDbFunctionExpressionData* f, KDbParseInfo* parseInfo) const override {
         Q_UNUSED(parseInfo);
         // Find type
         //! @todo Most likely but can be also other type
@@ -165,7 +169,7 @@ class MinMaxFunctionDeclaration : public BuiltInFunctionDeclaration
     Q_DECLARE_TR_FUNCTIONS(MinMaxFunctionDeclaration)
 public:
     MinMaxFunctionDeclaration() {}
-    virtual KDbField::Type returnType(const KDbFunctionExpressionData* f, KDbParseInfo* parseInfo) const {
+    KDbField::Type returnType(const KDbFunctionExpressionData* f, KDbParseInfo* parseInfo) const override {
         const KDbNArgExpressionData *argsData = f->args.constData()->convertConst<KDbNArgExpressionData>();
         if (argsData->children.isEmpty()) {
             return KDbField::Null;
@@ -274,8 +278,7 @@ class RandomFunctionDeclaration : public BuiltInFunctionDeclaration
     Q_DECLARE_TR_FUNCTIONS(RandomFunctionDeclaration)
 public:
     RandomFunctionDeclaration() {}
-    virtual KDbField::Type returnType(const KDbFunctionExpressionData* f, KDbParseInfo* parseInfo) const {
-        Q_UNUSED(parseInfo);
+    KDbField::Type returnType(const KDbFunctionExpressionData* f, KDbParseInfo* parseInfo) const override {
         const KDbNArgExpressionData *argsData = f->args.constData()->convertConst<KDbNArgExpressionData>();
         if (argsData->children.isEmpty()) {
             return KDbField::Double;
@@ -304,8 +307,10 @@ public:
             KDbField::Type t0;
             KDbField::Type t1;
             // deduce query parameter types
-            KDbQueryParameterExpressionData *queryParameterExpressionData0 = argsData->children.at(0)->convert<KDbQueryParameterExpressionData>();
-            KDbQueryParameterExpressionData *queryParameterExpressionData1 = argsData->children.at(1)->convert<KDbQueryParameterExpressionData>();
+            KDbQueryParameterExpressionData *queryParameterExpressionData0
+                = argsData->children.at(0)->convert<KDbQueryParameterExpressionData>();
+            KDbQueryParameterExpressionData *queryParameterExpressionData1
+                = argsData->children.at(1)->convert<KDbQueryParameterExpressionData>();
             if (queryParameterExpressionData0 && queryParameterExpressionData1) {
                 queryParameterExpressionData0->m_type = KDbField::Integer;
                 queryParameterExpressionData1->m_type = KDbField::Integer;
@@ -339,11 +344,12 @@ class CeilingFloorFunctionDeclaration : public BuiltInFunctionDeclaration
 {
 public:
     CeilingFloorFunctionDeclaration() {}
-    virtual KDbField::Type returnType(const KDbFunctionExpressionData* f, KDbParseInfo* parseInfo) const {
+    KDbField::Type returnType(const KDbFunctionExpressionData* f, KDbParseInfo* parseInfo) const override {
         Q_UNUSED(parseInfo);
         const KDbNArgExpressionData *argsData = f->args.constData()->convertConst<KDbNArgExpressionData>();
         if (argsData->children.count() == 1) {
-            KDbQueryParameterExpressionData *queryParameterExpressionData = argsData->children.at(0)->convert<KDbQueryParameterExpressionData>();
+            KDbQueryParameterExpressionData *queryParameterExpressionData
+                = argsData->children.at(0)->convert<KDbQueryParameterExpressionData>();
             if (queryParameterExpressionData) {
                 // Set query parameter type (if there are any) to deduced result type
                 //! @todo Most likely but can be also other type
@@ -371,12 +377,13 @@ private:
 
 //! A map of built-in SQL functions
 //! See https://community.kde.org/Kexi/Plugins/Queries/SQL_Functions for the status.
-class BuiltInFunctions : public QHash<QString, BuiltInFunctionDeclaration*>
+class BuiltInFunctions
 {
 public:
     BuiltInFunctions();
-    ~BuiltInFunctions() {
-        qDeleteAll(*this);
+    ~BuiltInFunctions()
+    {
+        qDeleteAll(m_functions);
     }
 
     //! @return function declaration's structure for name @a name
@@ -389,6 +396,7 @@ public:
 
     static int multipleArgs[];
 private:
+    QHash<QString, BuiltInFunctionDeclaration*> m_functions;
     QHash<QString, BuiltInFunctionDeclaration*> m_aliases;
     Q_DISABLE_COPY(BuiltInFunctions)
 };
@@ -396,7 +404,6 @@ private:
 int BuiltInFunctions::multipleArgs[] = { 0 };
 
 BuiltInFunctions::BuiltInFunctions()
-    : QHash<QString, BuiltInFunctionDeclaration*>()
 {
     BuiltInFunctionDeclaration *decl;
 #define _TYPES(name, ...) static int name[] = { __VA_ARGS__, KDbField::InvalidType }
@@ -420,9 +427,9 @@ BuiltInFunctions::BuiltInFunctions()
 #define _SIG0 \
     decl->signatures.push_back(sig0)
 
-    static int* sig0[] = { 0 };
+    static int* sig0[] = { nullptr };
 
-    insert(QLatin1String("ABS"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("ABS"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The abs(X) function returns the absolute value of the numeric argument X.
      Abs(X) returns NULL if X is NULL. Abs(X) returns 0.0 if X is a string or blob that
@@ -434,7 +441,7 @@ BuiltInFunctions::BuiltInFunctions()
     decl->copyReturnTypeFromArg = 0;
     _SIG(abs_1, argAnyNumberOrNull);
 
-    insert(QLatin1String("CEILING"), decl = new CeilingFloorFunctionDeclaration);
+    m_functions.insert(QLatin1String("CEILING"), decl = new CeilingFloorFunctionDeclaration);
     /* ceiling(X) returns the largest integer value not less than X. */
     // See also https://dev.mysql.com/doc/refman/5.1/en/mathematical-functions.html#function_ceiling
     // See also http://www.postgresql.org/docs/9.5/static/functions-math.html#FUNCTIONS-MATH-FUNC-TABLE
@@ -445,7 +452,7 @@ BuiltInFunctions::BuiltInFunctions()
     // result: 4, -99
     _SIG(ceiling, argAnyNumberOrNull);
 
-    insert(QLatin1String("CHAR"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("CHAR"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The char(X1,X2,...,XN) function returns a string composed of characters having
      the unicode code point values of integers X1 through XN, respectively. */
@@ -455,7 +462,7 @@ BuiltInFunctions::BuiltInFunctions()
     static int char_min_args[] = { 0 };
     _SIG(char_N, argAnyIntOrNull, multipleArgs, char_min_args);
 
-    insert(QLatin1String("COALESCE"), decl = new CoalesceFunctionDeclaration);
+    m_functions.insert(QLatin1String("COALESCE"), decl = new CoalesceFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The coalesce() function returns a copy of its first non-NULL argument, or NULL if
      all arguments are NULL. Coalesce() must have at least 2 arguments. */
@@ -464,7 +471,7 @@ BuiltInFunctions::BuiltInFunctions()
     static int coalesce_min_args[] = { 2 };
     _SIG(coalesce_N, argAnyOrNull, multipleArgs, coalesce_min_args);
 
-    insert(QLatin1String("FLOOR"), decl = new CeilingFloorFunctionDeclaration);
+    m_functions.insert(QLatin1String("FLOOR"), decl = new CeilingFloorFunctionDeclaration);
     /* floor(X) returns the largest integer value not greater than X. */
     // See also https://dev.mysql.com/doc/refman/5.1/en/mathematical-functions.html#function_floor
     // See also http://www.postgresql.org/docs/9.5/static/functions-math.html#FUNCTIONS-MATH-FUNC-TABLE
@@ -475,7 +482,7 @@ BuiltInFunctions::BuiltInFunctions()
     // result: 3, -100
     _SIG(floor, argAnyNumberOrNull);
 
-    insert(QLatin1String("GREATEST"), decl = new MinMaxFunctionDeclaration);
+    m_functions.insert(QLatin1String("GREATEST"), decl = new MinMaxFunctionDeclaration);
     m_aliases.insert(QLatin1String("MAX"), decl);
     // From https://www.sqlite.org/lang_corefunc.html
     // For SQLite MAX() is used.
@@ -507,7 +514,7 @@ BuiltInFunctions::BuiltInFunctions()
     static int greatest_min_args[] = { 2 };
     _SIG(greatest_N, argAnyOrNull, multipleArgs, greatest_min_args);
 
-    insert(QLatin1String("HEX"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("HEX"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     // See also https://dev.mysql.com/doc/refman/5.1/en/string-functions.html#function_hex
     /* The hex() function interprets its argument as a BLOB and returns a string which is
@@ -521,7 +528,7 @@ BuiltInFunctions::BuiltInFunctions()
     decl->defaultReturnType = KDbField::LongText;
     _SIG(hex_1, argAnyTextBLOBOrNull);
 
-    insert(QLatin1String("IFNULL"), decl = new CoalesceFunctionDeclaration);
+    m_functions.insert(QLatin1String("IFNULL"), decl = new CoalesceFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The ifnull() function returns a copy of its first non-NULL argument, or NULL if
      both arguments are NULL. Ifnull() must have exactly 2 arguments. The ifnull() function
@@ -531,7 +538,7 @@ BuiltInFunctions::BuiltInFunctions()
     // result: 17, NULL
     _SIG(ifnull_2, argAnyOrNull, argAnyOrNull);
 
-    insert(QLatin1String("INSTR"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("INSTR"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The instr(X,Y) function finds the first occurrence of string Y within string X and
      returns the number of prior characters plus 1, or 0 if Y is nowhere found within X.
@@ -549,7 +556,7 @@ BuiltInFunctions::BuiltInFunctions()
     decl->defaultReturnType = KDbField::Integer;
     _SIG(instr_2, argAnyTextOrNull, argAnyTextOrNull);
 
-    insert(QLatin1String("LEAST"), decl = new MinMaxFunctionDeclaration);
+    m_functions.insert(QLatin1String("LEAST"), decl = new MinMaxFunctionDeclaration);
     m_aliases.insert(QLatin1String("MIN"), decl);
     // From https://www.sqlite.org/lang_corefunc.html
     // For SQLite uses MIN().
@@ -577,7 +584,7 @@ BuiltInFunctions::BuiltInFunctions()
     static int least_min_args[] = { 2 };
     _SIG(least_N, argAnyOrNull, multipleArgs, least_min_args);
 
-    insert(QLatin1String("LENGTH"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("LENGTH"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     // See also https://dev.mysql.com/doc/refman/5.1/en/string-functions.html#function_length
     /* For a string value X, the length(X) function returns the number of characters (not
@@ -596,7 +603,7 @@ BuiltInFunctions::BuiltInFunctions()
     decl->defaultReturnType = KDbField::Integer;
     _SIG(length_1, argAnyTextBLOBOrNull);
 
-    insert(QLatin1String("LOWER"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("LOWER"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The lower(X) function returns a copy of string X with all characters converted
      to lower case. */
@@ -609,7 +616,7 @@ BuiltInFunctions::BuiltInFunctions()
     decl->defaultReturnType = KDbField::LongText;
     _SIG(lower_1, argAnyTextOrNull);
 
-    insert(QLatin1String("LTRIM"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("LTRIM"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The ltrim(X,Y) function returns a string formed by removing any and all characters
      that appear in Y from the left side of X. If the Y argument is omitted, ltrim(X)
@@ -626,7 +633,7 @@ BuiltInFunctions::BuiltInFunctions()
     _SIG(ltrim_1, argAnyTextOrNull);
     _SIG(ltrim_2, argAnyTextOrNull, argAnyTextOrNull);
 
-    insert(QLatin1String("NULLIF"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("NULLIF"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The nullif(X,Y) function returns its first argument if the arguments are different
      and NULL if the arguments are the same. The nullif(X,Y) function searches its
@@ -640,7 +647,7 @@ BuiltInFunctions::BuiltInFunctions()
     decl->copyReturnTypeFromArg = 0;
     _SIG(nullif_2, argAnyOrNull, argAnyOrNull);
 
-    insert(QLatin1String("RANDOM"), decl = new RandomFunctionDeclaration);
+    m_functions.insert(QLatin1String("RANDOM"), decl = new RandomFunctionDeclaration);
     /* RANDOM() returns a random floating-point value v in the range 0 <= v < 1.0.
      RANDOM(X,Y) - returns returns a random integer that is equal or greater than X
      and less than Y. */
@@ -664,7 +671,7 @@ BuiltInFunctions::BuiltInFunctions()
     _SIG0;
     _SIG(random_2, argAnyIntOrNull, argAnyIntOrNull);
 
-    insert(QLatin1String("ROUND"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("ROUND"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The round(X,Y) function returns a floating-point value X rounded to Y digits to the
      right of the decimal point. If the Y argument is omitted, it is assumed to be 0. */
@@ -677,7 +684,7 @@ BuiltInFunctions::BuiltInFunctions()
     _SIG(round_1, argAnyNumberOrNull);
     _SIG(round_2, argAnyNumberOrNull, argAnyIntOrNull);
 
-    insert(QLatin1String("RTRIM"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("RTRIM"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The rtrim(X,Y) function returns a string formed by removing any and all characters
      that appear in Y from the right side of X. If the Y argument is omitted, rtrim(X)
@@ -694,21 +701,21 @@ BuiltInFunctions::BuiltInFunctions()
     _SIG(rtrim_1, argAnyTextOrNull);
     _SIG(rtrim_2, argAnyTextOrNull, argAnyTextOrNull);
 
-    insert(QLatin1String("SOUNDEX"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("SOUNDEX"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The soundex(X) function returns a string that is the soundex encoding of the string
      X. The string "?000" is returned if the argument is NULL or contains non-ASCII
      alphabetic characters. */
     // See also https://dev.mysql.com/doc/refman/5.1/en/string-functions.html#function_soundex
     // See also http://www.postgresql.org/docs/9.5/static/fuzzystrmatch.html#AEN165853
-    //! @todo we call drv_executeVoidSQL("CREATE EXTENSION IF NOT EXISTS fuzzystrmatch") on connection,
+    //! @todo we call drv_executeSql("CREATE EXTENSION IF NOT EXISTS fuzzystrmatch") on connection,
     //!       do that on first use of SOUNDEX()
     // example: SELECT SOUNDEX("John")
     // result: "J500"
     decl->defaultReturnType = KDbField::Text;
     _SIG(soundex, argAnyTextOrNull);
 
-    insert(QLatin1String("SUBSTR"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("SUBSTR"), decl = new BuiltInFunctionDeclaration);
     // From https://www.sqlite.org/lang_corefunc.html
     /* The substr(X,Y) returns all characters through the end of the string X beginning with
     the Y-th. The left-most character of X is number 1. If Y is negative then the
@@ -722,7 +729,7 @@ BuiltInFunctions::BuiltInFunctions()
     _SIG(substr_3, argAnyTextOrNull, argAnyIntOrNull, argAnyIntOrNull);
     decl->copyReturnTypeFromArg = 0;
 
-     insert(QLatin1String("TRIM"), decl = new BuiltInFunctionDeclaration);
+     m_functions.insert(QLatin1String("TRIM"), decl = new BuiltInFunctionDeclaration);
      // From https://www.sqlite.org/lang_corefunc.html
      /* The trim(X,Y) function returns a string formed by removing any and all characters
       that appear in Y from both ends of X. If the Y argument is omitted, trim(X) removes
@@ -739,7 +746,7 @@ BuiltInFunctions::BuiltInFunctions()
      _SIG(trim_1, argAnyTextOrNull);
      _SIG(trim_2, argAnyTextOrNull, argAnyTextOrNull);
 
-     insert(QLatin1String("UNICODE"), decl = new BuiltInFunctionDeclaration);
+     m_functions.insert(QLatin1String("UNICODE"), decl = new BuiltInFunctionDeclaration);
      // From https://www.sqlite.org/lang_corefunc.html
      /* The unicode(X) function returns the numeric unicode code point corresponding to
       the first character of the string X. If the argument to unicode(X) is not a string
@@ -751,7 +758,7 @@ BuiltInFunctions::BuiltInFunctions()
      decl->defaultReturnType = KDbField::Integer;
      _SIG(unicode_1, argAnyTextOrNull);
 
-     insert(QLatin1String("UPPER"), decl = new BuiltInFunctionDeclaration);
+     m_functions.insert(QLatin1String("UPPER"), decl = new BuiltInFunctionDeclaration);
      // From https://www.sqlite.org/lang_corefunc.html
      /* The upper(X) function returns a copy of string X with all characters converted
       to upper case. */
@@ -765,7 +772,7 @@ BuiltInFunctions::BuiltInFunctions()
      _SIG(upper_1, argAnyTextOrNull);
 
 #ifdef KDB_ENABLE_SQLITE_SPECIFIC_FUNCTIONS
-    insert(QLatin1String("GLOB"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("GLOB"), decl = new BuiltInFunctionDeclaration);
     //! @todo GLOB(X,Y) is SQLite-specific and is not present in MySQL so we don't expose it; use GLOB operator instead.
     //! We may want to address it in raw SQL generation time.
     // From https://www.sqlite.org/lang_corefunc.html
@@ -777,7 +784,7 @@ BuiltInFunctions::BuiltInFunctions()
     decl->defaultReturnType = KDbField::Boolean;
     _SIG(glob_2, argAnyTextOrNull, argAnyOrNull /* will be casted to text */);
 
-    insert(QLatin1String("LIKE"), decl = new BuiltInFunctionDeclaration);
+    m_functions.insert(QLatin1String("LIKE"), decl = new BuiltInFunctionDeclaration);
     //! @todo LIKE(X,Y,[Z]) not present in MySQL so we don't expose it; use LIKE operator instead.
     //! We may want to address it in raw SQL generation time.
     // From https://www.sqlite.org/lang_corefunc.html
@@ -793,7 +800,7 @@ BuiltInFunctions::BuiltInFunctions()
 
 BuiltInFunctionDeclaration* BuiltInFunctions::value(const QString &name) const
 {
-    BuiltInFunctionDeclaration* f = QHash<QString, BuiltInFunctionDeclaration*>::value(name);
+    BuiltInFunctionDeclaration* f = m_functions.value(name);
     if (!f) {
         f = m_aliases.value(name);
     }
@@ -846,7 +853,7 @@ void KDbFunctionExpressionData::debugInternal(QDebug dbg, KDb::ExpressionCallSta
         dbg.nospace() << ',';
         args.data()->debug(dbg, callStack);
     }
-    dbg.nospace() << QString::fromLatin1(",type=%1)").arg(KDbDriver::defaultSQLTypeName(type()));
+    dbg.nospace() << QString::fromLatin1(",type=%1)").arg(KDbDriver::defaultSqlTypeName(type()));
 }
 
 static QByteArray greatestOrLeastName(const QByteArray &name)
@@ -919,7 +926,7 @@ KDbField::Type KDbFunctionExpressionData::typeInternal(KDb::ExpressionCallStack*
     Q_UNUSED(callStack);
     const BuiltInFunctionDeclaration *decl = _builtInFunctions->value(name);
     if (decl) {
-        return decl->returnType(this, 0);
+        return decl->returnType(this, nullptr);
     }
     //! @todo
     return KDbField::InvalidType;
@@ -1161,7 +1168,7 @@ bool KDbFunctionExpressionData::validateInternal(KDbParseInfo *parseInfo,
     std::vector<int> argCounts;
     int i = 0;
     argCounts.resize(decl->signatures.size());
-    int **signature = 0;
+    int **signature = nullptr;
     bool multipleArgs = false; // special case, e.g. for CHARS(v1, ... vN)
     for(std::vector<int**>::const_iterator it(decl->signatures.begin());
         it != decl->signatures.end(); ++it, ++i)
@@ -1338,13 +1345,13 @@ KDbFunctionExpression::~KDbFunctionExpression()
 // static
 bool KDbFunctionExpression::isBuiltInAggregate(const QString& function)
 {
-    return _builtInAggregates->contains(function.toUpper());
+    return _builtInAggregates->data.contains(function.toUpper());
 }
 
 // static
 QStringList KDbFunctionExpression::builtInAggregates()
 {
-    return _builtInAggregates->toList();
+    return _builtInAggregates->data.toList();
 }
 
 //static
@@ -1389,16 +1396,16 @@ KDbEscapedString KDbFunctionExpression::greatestOrLeastFunctionUsingCaseToString
 {
     // (CASE WHEN (v0) IS NULL OR .. OR (vN) IS NULL THEN NULL ELSE F(v0,..,vN) END)
     if (args.argCount() >= 2) {
-        KDbEscapedString whenSQL;
-        whenSQL.reserve(256);
+        KDbEscapedString whenSql;
+        whenSql.reserve(256);
         foreach(const ExplicitlySharedExpressionDataPointer &child, args.d.constData()->children) {
-            if (!whenSQL.isEmpty()) {
-                whenSQL += " OR ";
+            if (!whenSql.isEmpty()) {
+                whenSql += " OR ";
             }
-            whenSQL += QLatin1Char('(') + child->toString(driver, params, callStack)
+            whenSql += QLatin1Char('(') + child->toString(driver, params, callStack)
                     + QLatin1String(") IS NULL");
         }
-        return KDbEscapedString("(CASE WHEN (") + whenSQL
+        return KDbEscapedString("(CASE WHEN (") + whenSql
                + QLatin1String(") THEN NULL ELSE (")
                + KDbFunctionExpression::toString(name, driver, args, params, callStack)
                + QLatin1String(") END)");
