@@ -1,5 +1,5 @@
 /* This file is part of the KDE project
-   Copyright (C) 2004-2016 Jarosław Staniek <staniek@kde.org>
+   Copyright (C) 2004-2017 Jarosław Staniek <staniek@kde.org>
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
@@ -21,16 +21,17 @@
 #include "KDb.h"
 #include "KDbConnection.h"
 #include "KDbTableSchema.h"
+#include "KDbQueryAsterisk.h"
 #include "KDbQuerySchema.h"
+#include "KDbQuerySchema_p.h"
+#include "KDbOrderByColumn.h"
 #include "kdb_debug.h"
 #include "generated/sqlparser.h"
 
 #include <QMutableListIterator>
 
-#include <assert.h>
-
-KDbParser *globalParser = 0;
-KDbField *globalField = 0;
+KDbParser *globalParser = nullptr;
+KDbField *globalField = nullptr;
 QList<KDbField*> fieldList;
 int globalCurrentPos = 0;
 QByteArray globalToken;
@@ -40,7 +41,7 @@ extern int yylex_destroy(void);
 //-------------------------------------
 
 KDbParserPrivate::KDbParserPrivate()
-    : table(0), query(0), connection(0), initialized(false)
+    : table(nullptr), query(nullptr), connection(nullptr), initialized(false)
 {
     reset();
 }
@@ -56,9 +57,9 @@ void KDbParserPrivate::reset()
     sql.clear();
     error = KDbParserError();
     delete table;
-    table = 0;
+    table = nullptr;
     delete query;
-    query = 0;
+    query = nullptr;
 }
 
 void KDbParserPrivate::setStatementType(KDbParser::StatementType type)
@@ -79,10 +80,17 @@ void KDbParserPrivate::setTableSchema(KDbTableSchema *table)
 
 void KDbParserPrivate::setQuerySchema(KDbQuerySchema *query)
 {
-    delete this->query;
+    if (this->query != query) {
+        delete this->query;
+    }
     this->query = query;
 }
 
+KDbQuerySchema* KDbParserPrivate::createQuery()
+{
+    return query ? query : new KDbQuerySchema;
+}
+
 //-------------------------------------
 
 KDbParseInfo::KDbParseInfo(KDbQuerySchema *query)
@@ -98,8 +106,23 @@ KDbParseInfo::~KDbParseInfo()
 
 QList<int> KDbParseInfo::tablesAndAliasesForName(const QString &tableOrAliasName) const
 {
+    QList<int> result;
     const QList<int> *list = d->repeatedTablesAndAliases.value(tableOrAliasName);
-    return list ? *list : QList<int>();
+    if (list) {
+        result = *list;
+    }
+    if (result.isEmpty()) {
+        int position = d->querySchema->tablePositionForAlias(tableOrAliasName);
+        if (position == -1) {
+            position = d->querySchema->tablePosition(tableOrAliasName);
+            if (position != -1) {
+                result.append(position);
+            }
+        } else {
+            result.append(position);
+        }
+    }
+    return result;
 }
 
 KDbQuerySchema* KDbParseInfo::querySchema() const
@@ -161,7 +184,8 @@ void yyerror(const char *str)
 
     const bool otherError = (qstrnicmp(str, "other error", 11) == 0);
     const bool syntaxError = qstrnicmp(str, "syntax error", 12) == 0;
-    if ((   globalParser->error().type().isEmpty() && (str == 0 || strlen(str) == 0 || syntaxError))
+    if ((globalParser->error().type().isEmpty()
+         && (str == nullptr || strlen(str) == 0 || syntaxError))
         || otherError)
     {
         kdbDebug() << globalParser->statement();
@@ -185,9 +209,9 @@ void yyerror(const char *str)
 
         //! @todo exact invalid expression can be selected in the editor, based on KDbParseInfo data
         if (!otherError) {
-            const bool isKDbSQLKeyword = KDb::isKDbSQLKeyword(globalToken);
-            if (isKDbSQLKeyword || syntaxError) {
-                if (isKDbSQLKeyword) {
+            const bool isKDbSqlKeyword = KDb::isKDbSqlKeyword(globalToken);
+            if (isKDbSqlKeyword || syntaxError) {
+                if (isKDbSqlKeyword) {
                     KDbParserPrivate::get(globalParser)->setError(KDbParserError(KDbParser::tr("Syntax Error"),
                                                           KDbParser::tr("\"%1\" is a reserved keyword.").arg(QLatin1String(globalToken)),
                                                           globalToken, globalCurrentPos));
@@ -221,27 +245,23 @@ void setError(const QString& errDesc)
 
 //! @internal Parses @a data for parser @a p
 //! @todo Make it REENTRANT
-bool parseData(KDbParser *p, const KDbEscapedString &sql)
+bool parseData()
 {
-    globalParser = p;
-    globalParser->reset();
-    globalField = 0;
     fieldList.clear();
 
+    const KDbEscapedString sql(globalParser->statement());
     if (sql.isEmpty()) {
         KDbParserError err(KDbParser::tr("Error"),
                            KDbParser::tr("No query statement specified."),
                            globalToken, globalCurrentPos);
         KDbParserPrivate::get(globalParser)->setError(err);
         yyerror("");
-        globalParser = 0;
         return false;
     }
 
     const char *data = sql.constData();
     tokenize(data);
     if (!globalParser->error().type().isEmpty()) {
-        globalParser = 0;
         return false;
     }
 
@@ -269,7 +289,6 @@ bool parseData(KDbParser *p, const KDbEscapedString &sql)
         ok = false;
     }
     yylex_destroy();
-    globalParser = 0;
     return ok;
 }
 
@@ -300,7 +319,7 @@ bool addColumn(KDbParseInfo *parseInfo, const KDbExpression &columnExpr)
                 return false;
             }
         } else if (v_e.tableForQueryAsterisk()) {//one-table asterisk
-            KDbQueryAsterisk *a = new KDbQueryAsterisk(parseInfo->querySchema(), v_e.tableForQueryAsterisk());
+            KDbQueryAsterisk *a = new KDbQueryAsterisk(parseInfo->querySchema(), *v_e.tableForQueryAsterisk());
             if (!parseInfo->querySchema()->addAsterisk(a)) {
                 delete a;
                 setError(KDbParser::tr("\"<table>.*\" could not be added."));
@@ -368,11 +387,19 @@ KDbQuerySchema* buildSelectQuery(
             KDbTableSchema *s = globalParser->connection()->tableSchema(tname);
             if (!s) {
                 setError(KDbParser::tr("Table \"%1\" does not exist.").arg(tname));
-                return 0;
+                return nullptr;
             }
             QString tableOrAliasName = KDb::iifNotEmpty(aliasString, tname);
             if (!aliasString.isEmpty()) {
 //    kdbDebug() << "- add alias for table: " << aliasString;
+                const int tablePosition = querySchema->tablePositionForAlias(aliasString);
+                if (tablePosition != -1 && tablePosition != i) {
+                    KDbTableSchema* tableForAlias = querySchema->tables()->at(tablePosition);
+                    setError(KDbParser::tr("Could not set alias \"%1\" for table \"%2\". "
+                                           "This alias is already set for table \"%3\".")
+                             .arg(aliasString).arg(tname).arg(tableForAlias->name()));
+                    break;
+                }
             }
             // 1. collect information about first repeated table name or alias
             //    (potential ambiguity)
@@ -401,7 +428,7 @@ KDbQuerySchema* buildSelectQuery(
                 aliasVariable = e.toBinary().right().toVariable();
                 if (aliasVariable.isNull()) {
                     setError(KDbParser::tr("Invalid alias definition for column \"%1\".")
-                                           .arg(columnExpr.toString(0).toString())); //ok?
+                                           .arg(columnExpr.toString(nullptr).toString())); //ok?
                     break;
                 }
             }
@@ -414,13 +441,14 @@ KDbQuerySchema* buildSelectQuery(
                 || c == KDb::LogicalExpression
                 || c == KDb::RelationalExpression
                 || c == KDb::FunctionExpression
-                || c == KDb::AggregationExpression;
+                || c == KDb::AggregationExpression
+                || c == KDb::QueryParameterExpression;
 
             if (c == KDb::VariableExpression) {
                 if (columnExpr.toVariable().name() == QLatin1String("*")) {
                     if (containsAsteriskColumn) {
                         setError(KDbParser::tr("More than one asterisk \"*\" is not allowed."));
-                        return 0;
+                        return nullptr;
                     }
                     else {
                         containsAsteriskColumn = true;
@@ -434,7 +462,7 @@ KDbQuerySchema* buildSelectQuery(
 //! @todo IMPORTANT: it.remove();
             } else if (aliasVariable.isNull()) {
                 setError(KDbParser::tr("Invalid \"%1\" column definition.")
-                                       .arg(e.toString(0).toString())); //ok?
+                                       .arg(e.toString(nullptr).toString())); //ok?
                 break;
             }
             else {
@@ -449,12 +477,22 @@ KDbQuerySchema* buildSelectQuery(
             if (!aliasVariable.isNull()) {
 //    kdbDebug() << "ALIAS \"" << aliasVariable->name << "\" set for column "
 //     << columnNum;
-                querySchema->setColumnAlias(columnNum, aliasVariable.name());
+                const int currentColumn = querySchema->columnPositionForAlias(aliasVariable.name());
+                if (currentColumn != -1) {
+                    setError(KDbParser::tr("Could not set alias \"%1\" for column #%2. This alias is already set for column #%3.")
+                             .arg(aliasVariable.name()).arg(columnNum + 1).arg(currentColumn + 1));
+                    break;
+                }
+                if (!querySchema->setColumnAlias(columnNum, aliasVariable.name())) {
+                    setError(KDbParser::tr("Could not set alias \"%1\" for column #%2.")
+                             .arg(aliasVariable.name()).arg(columnNum + 1));
+                    break;
+                }
             }
         } // for
         if (!globalParser->error().message().isEmpty()) { // we could not return earlier (inside the loop)
                                                           // because we want run CLEANUP what could crash QMutableListIterator.
-            return 0;
+            return nullptr;
         }
     }
     //----- SELECT options
@@ -463,9 +501,9 @@ KDbQuerySchema* buildSelectQuery(
         if (!options->whereExpr.isNull()) {
             if (!options->whereExpr.validate(&parseInfo)) {
                 setError(parseInfo.errorMessage(), parseInfo.errorDescription());
-                return 0;
+                return nullptr;
             }
-            querySchema->setWhereExpression(options->whereExpr);
+            KDbQuerySchemaPrivate::setWhereExpressionInternal(querySchema, options->whereExpr);
         }
         //----- ORDER BY
         if (options->orderByColumns) {
@@ -476,19 +514,23 @@ KDbQuerySchema* buildSelectQuery(
             for (;count > 0; --it, --count)
                 /*opposite direction due to parser specifics*/
             {
-                //first, try to find a column name or alias (outside of asterisks)
-                KDbQueryColumnInfo *columnInfo = querySchema->columnInfo((*it).aliasOrName, false/*outside of asterisks*/);
+                // first, try to find a column name or alias (outside of asterisks)
+                KDbQueryColumnInfo *columnInfo = querySchema->columnInfo(
+                    globalParser->connection(), (*it).aliasOrName,
+                    KDbQuerySchema::ExpandMode::Unexpanded /*outside of asterisks*/);
                 if (columnInfo) {
-                    orderByColumnList->appendColumn(columnInfo, (*it).ascending);
+                    orderByColumnList->appendColumn(columnInfo, (*it).order);
                 } else {
                     //failed, try to find a field name within all the tables
                     if ((*it).columnNumber != -1) {
-                        if (!orderByColumnList->appendColumn(querySchema,
-                                                            (*it).ascending, (*it).columnNumber - 1)) {
+                        if (!orderByColumnList->appendColumn(globalParser->connection(),
+                                                             querySchema, (*it).order,
+                                                             (*it).columnNumber - 1))
+                        {
                             setError(KDbParser::tr("Could not define sorting. Column at "
                                                    "position %1 does not exist.")
                                                    .arg((*it).columnNumber));
-                            return 0;
+                            return nullptr;
                         }
                     } else {
                         KDbField * f = querySchema->findTableField((*it).aliasOrName);
@@ -496,9 +538,9 @@ KDbQuerySchema* buildSelectQuery(
                             setError(KDbParser::tr("Could not define sorting. "
                                                    "Column name or alias \"%1\" does not exist.")
                                                    .arg((*it).aliasOrName));
-                            return 0;
+                            return nullptr;
                         }
-                        orderByColumnList->appendField(f, (*it).ascending);
+                        orderByColumnList->appendField(f, (*it).order);
                     }
                 }
             }
