@@ -71,6 +71,7 @@ cat << EOF > generated/sqlparser.h
 
 #include "KDbExpression.h"
 #include "KDbField.h"
+#include "KDbOrderByColumn.h"
 
 struct OrderByColumnInternal;
 struct SelectOptionsInternal;
@@ -138,7 +139,7 @@ cat << EOF > generated/KDbToken.h
 
 class KDbDriver;
 
-/*! @brief A type-safe KDBSQL token
+/*! @brief A type-safe KDbSQL token
  It can be used in KDb expressions
  @see KDbExpression */
 class KDB_EXPORT KDbToken
@@ -240,6 +241,7 @@ done >> generated/KDbToken.h
 cat << EOF >> generated/KDbToken.h
     // -- end of constants --
 
+    class List;
 private:
     inline KDbToken(int value) : v(value) {}
     int v;
@@ -277,10 +279,13 @@ cat << EOF > generated/KDbToken.cpp
 
 #include "KDbToken.h"
 #include "KDbDriver.h"
+#include "KDbDriver_p.h"
 #include "KDbDriverBehavior.h"
 #include "sqlparser.h"
 #include "parser/KDbParser_p.h"
 
+#include <QGlobalStatic>
+
 KDbToken::KDbToken(char charToken)
     : v(g_tokenName(charToken) == nullptr ? 0 : charToken)
 {
@@ -317,10 +322,10 @@ QString KDbToken::toString(const KDbDriver *driver) const
     case ::NOT_EQUAL2: return QLatin1String("!=");
     case ::LESS_OR_EQUAL: return QLatin1String("<=");
     case ::GREATER_OR_EQUAL: return QLatin1String(">=");
-    case ::LIKE: return driver ? driver->behavior()->LIKE_OPERATOR : QLatin1String("LIKE");
+    case ::LIKE: return driver ? KDbDriverPrivate::behavior(driver)->LIKE_OPERATOR : QLatin1String("LIKE");
     case ::NOT_LIKE:
         return driver
-            ? (QString::fromLatin1("NOT ") + driver->behavior()->LIKE_OPERATOR)
+            ? (QString::fromLatin1("NOT ") + KDbDriverPrivate::behavior(driver)->LIKE_OPERATOR)
             : QString::fromLatin1("NOT LIKE");
     case ::SQL_IN: return QLatin1String("IN");
         // other logical operations: OR (or ||) AND (or &&) XOR
@@ -351,20 +356,29 @@ KDB_EXPORT QDebug operator<<(QDebug dbg, KDbToken token)
     return dbg.space();
 }
 
-static QList<KDbToken> g_allTokens;
+//! @internal
+class KDbToken::List
+{
+public:
+    List()
+    {
+        for (int i = 0; i < KDbToken::maxTokenValue; ++i) {
+            if (g_tokenName(i)) {
+                data.append(KDbToken(i));
+            }
+        }
+    }
+    QList<KDbToken> data;
+};
+
+Q_GLOBAL_STATIC(KDbToken::List, g_allTokens)
 
 //static
 QList<KDbToken> KDbToken::allTokens()
 {
-    if (g_allTokens.isEmpty()) {
-        for (int i = 0; i < KDbToken::maxTokenValue; ++i) {
-            if (g_tokenName(i)) {
-                g_allTokens.append(i);
-            }
-        }
-    }
-    return g_allTokens;
+    return g_allTokens->data;
 }
+
 EOF
 
 extractTokens | while read token value; do
