@@ -1,7 +1,7 @@
 /* This file is part of the KDE project
    Copyright (C) 2002 Lucijan Busch <lucijan@gmx.at>
    Copyright (C) 2003 Daniel Molkentin <molkentin@kde.org>
-   Copyright (C) 2003-2014 Jarosław Staniek <staniek@kde.org>
+   Copyright (C) 2003-2017 Jarosław Staniek <staniek@kde.org>
    Copyright (C) 2014 Michał Poteralski <michalpoteralskikde@gmail.com>
 
    This program is free software; you can redistribute it and/or
@@ -29,6 +29,7 @@
 #include "KDbCursor.h"
 #include "KDbError.h"
 #include "KDb.h"
+#include "KDbOrderByColumn.h"
 #include "KDbQuerySchema.h"
 #include "KDbRecordEditBuffer.h"
 #include "KDbTableViewColumn.h"
@@ -55,7 +56,7 @@ public:
         m_collator = icu::Collator::createInstance(status);
         if (U_FAILURE(status)) {
             kdbWarning() << "Could not create instance of collator:" << status;
-            m_collator = 0;
+            m_collator = nullptr;
         } else {
             // enable normalization by default
             m_collator->setAttribute(UCOL_NORMALIZATION_MODE, UCOL_ON, status);
@@ -83,7 +84,7 @@ Q_GLOBAL_STATIC(CollatorInstance, KDb_collator)
 class LessThanFunctor
 {
 private:
-    bool m_ascendingOrder;
+    KDbOrderByColumn::SortOrder m_order;
     QVariant m_leftTmp, m_rightTmp;
     int m_sortColumn;
 
@@ -128,13 +129,13 @@ private:
         const QString &as = left.toString();
         const QString &bs = right.toString();
 
-        const QChar *a = as.isEmpty() ? 0 : as.unicode();
-        const QChar *b = bs.isEmpty() ? 0 : bs.unicode();
+        const QChar *a = as.isEmpty() ? nullptr : as.unicode();
+        const QChar *b = bs.isEmpty() ? nullptr : bs.unicode();
 
-        if (a == 0) {
-            return b != 0;
+        if (a == nullptr) {
+            return b != nullptr;
         }
-        if (a == b || b == 0) {
+        if (a == b || b == nullptr) {
             return false;
         }
 
@@ -172,9 +173,9 @@ private:
 
 public:
     LessThanFunctor()
-            : m_ascendingOrder(true)
+            : m_order(KDbOrderByColumn::SortOrder::Ascending)
             , m_sortColumn(-1)
-            , m_lessThanFunction(0)
+            , m_lessThanFunction(nullptr)
     {
     }
 
@@ -213,8 +214,8 @@ public:
         }
     }
 
-    void setAscendingOrder(bool ascending) {
-        m_ascendingOrder = ascending;
+    void setSortOrder(KDbOrderByColumn::SortOrder order) {
+        m_order = order;
     }
 
     void setSortColumn(int column) {
@@ -227,26 +228,28 @@ public:
     bool operator()(KDbRecordData* record1, KDbRecordData* record2) {
         // compare NULLs : NULL is smaller than everything
         if ((m_leftTmp = record1->at(m_sortColumn)).isNull())
-            return _IIF(m_ascendingOrder, !record2->at(m_sortColumn).isNull());
+            return _IIF(m_order == KDbOrderByColumn::SortOrder::Ascending,
+                        !record2->at(m_sortColumn).isNull());
         if ((m_rightTmp = record2->at(m_sortColumn)).isNull())
-            return !m_ascendingOrder;
+            return m_order == KDbOrderByColumn::SortOrder::Descending;
 
-        return _IIF(m_ascendingOrder, m_lessThanFunction(m_leftTmp, m_rightTmp));
+        return _IIF(m_order == KDbOrderByColumn::SortOrder::Ascending,
+                    m_lessThanFunction(m_leftTmp, m_rightTmp));
     }
 };
 #undef _IIF
 #undef CAST_AND_COMPARE
 
 //! @internal
-class KDbTableViewData::Private
+class Q_DECL_HIDDEN KDbTableViewData::Private
 {
 public:
     Private()
             : sortColumn(0)
             , realSortColumn(0)
-            , sortOrder(Qt::AscendingOrder)
+            , sortOrder(KDbOrderByColumn::SortOrder::Ascending)
             , type(1)
-            , pRecordEditBuffer(0)
+            , pRecordEditBuffer(nullptr)
             , readOnly(false)
             , insertingEnabled(true)
             , containsRecordIdInfo(false)
@@ -274,7 +277,7 @@ public:
     int realSortColumn;
 
     //! Specifies sorting order
-    Qt::SortOrder sortOrder;
+    KDbOrderByColumn::SortOrder sortOrder;
 
     LessThanFunctor lessThanFunctor;
 
@@ -296,7 +299,7 @@ public:
     //! @see KDbTableViewData::containsRecordIdInfo()
     bool containsRecordIdInfo;
 
-    int autoIncrementedColumn;
+    mutable int autoIncrementedColumn;
 };
 
 //-------------------------------
@@ -306,7 +309,8 @@ KDbTableViewData::KDbTableViewData()
         , KDbTableViewDataBase()
         , d(new Private)
 {
-    init();
+    d->realColumnCount = 0;
+    d->cursor = nullptr;
 }
 
 // db-aware ctor
@@ -315,28 +319,30 @@ KDbTableViewData::KDbTableViewData(KDbCursor *c)
         , KDbTableViewDataBase()
         , d(new Private)
 {
-    init();
     d->cursor = c;
     d->containsRecordIdInfo = d->cursor->containsRecordIdInfo();
     if (d->cursor && d->cursor->query()) {
-        const KDbQuerySchema::FieldsExpandedOptions fieldsExpandedOptions
-        = d->containsRecordIdInfo ? KDbQuerySchema::WithInternalFieldsAndRecordId
-          : KDbQuerySchema::WithInternalFields;
-        d->realColumnCount = d->cursor->query()->fieldsExpanded(fieldsExpandedOptions).count();
+        const KDbQuerySchema::FieldsExpandedMode fieldsExpandedMode
+        = d->containsRecordIdInfo ? KDbQuerySchema::FieldsExpandedMode::WithInternalFieldsAndRecordId
+                                  : KDbQuerySchema::FieldsExpandedMode::WithInternalFields;
+        d->realColumnCount = d->cursor->query()->fieldsExpanded(
+                    d->cursor->connection(), fieldsExpandedMode).count();
     } else {
         d->realColumnCount = d->columns.count() + (d->containsRecordIdInfo ? 1 : 0);
     }
 
     // Allocate KDbTableViewColumn objects for each visible query column
-    const KDbQueryColumnInfo::Vector fields = d->cursor->query()->fieldsExpanded();
+    const KDbQueryColumnInfo::Vector fields
+            = d->cursor->query()->fieldsExpanded(d->cursor->connection());
     const int fieldCount = fields.count();
     for (int i = 0;i < fieldCount;i++) {
         KDbQueryColumnInfo *ci = fields[i];
-        if (ci->visible) {
-            KDbQueryColumnInfo *visibleLookupColumnInfo = 0;
+        if (ci->isVisible()) {
+            KDbQueryColumnInfo *visibleLookupColumnInfo = nullptr;
             if (ci->indexForVisibleLookupValue() != -1) {
-                //Lookup field is defined
-                visibleLookupColumnInfo = d->cursor->query()->expandedOrInternalField(ci->indexForVisibleLookupValue());
+                // Lookup field is defined
+                visibleLookupColumnInfo = d->cursor->query()->expandedOrInternalField(
+                    d->cursor->connection(), ci->indexForVisibleLookupValue());
             }
             KDbTableViewColumn* col = new KDbTableViewColumn(*d->cursor->query(), ci, visibleLookupColumnInfo);
             addColumn(col);
@@ -344,47 +350,20 @@ KDbTableViewData::KDbTableViewData(KDbCursor *c)
     }
 }
 
-KDbTableViewData::KDbTableViewData(
-    const QList<QVariant> &keys, const QList<QVariant> &values,
-    KDbField::Type keyType, KDbField::Type valueType)
-        : QObject()
-        , KDbTableViewDataBase()
-        , d(new Private)
+KDbTableViewData::KDbTableViewData(const QList<QVariant> &keys, const QList<QVariant> &values,
+                                   KDbField::Type keyType, KDbField::Type valueType)
+    : KDbTableViewData()
 {
-    init(keys, values, keyType, valueType);
-}
-
-KDbTableViewData::KDbTableViewData(
-    KDbField::Type keyType, KDbField::Type valueType)
-        : QObject()
-        , KDbTableViewDataBase()
-        , d(new Private)
-{
-    const QList<QVariant> empty;
-    init(empty, empty, keyType, valueType);
-}
-
-KDbTableViewData::~KDbTableViewData()
-{
-    emit destroying();
-    clearInternal(false /* !processEvents */);
-    qDeleteAll(d->columns);
-    delete d;
-}
-
-void KDbTableViewData::init(
-    const QList<QVariant> &keys, const QList<QVariant> &values,
-    KDbField::Type keyType, KDbField::Type valueType)
-{
-    init();
     KDbField *keyField = new KDbField(QLatin1String("key"), keyType);
     keyField->setPrimaryKey(true);
-    KDbTableViewColumn *keyColumn = new KDbTableViewColumn(keyField, true);
+    KDbTableViewColumn *keyColumn
+        = new KDbTableViewColumn(keyField, KDbTableViewColumn::FieldIsOwned::Yes);
     keyColumn->setVisible(false);
     addColumn(keyColumn);
 
     KDbField *valueField = new KDbField(QLatin1String("value"), valueType);
-    KDbTableViewColumn *valueColumn = new KDbTableViewColumn(valueField, true);
+    KDbTableViewColumn *valueColumn
+        = new KDbTableViewColumn(valueField, KDbTableViewColumn::FieldIsOwned::Yes);
     addColumn(valueColumn);
 
     int cnt = qMin(keys.count(), values.count());
@@ -398,15 +377,22 @@ void KDbTableViewData::init(
     }
 }
 
-void KDbTableViewData::init()
+KDbTableViewData::KDbTableViewData(KDbField::Type keyType, KDbField::Type valueType)
+    : KDbTableViewData(QList<QVariant>(), QList<QVariant>(), keyType, valueType)
 {
-    d->realColumnCount = 0;
-    d->cursor = 0;
+}
+
+KDbTableViewData::~KDbTableViewData()
+{
+    emit destroying();
+    clearInternal(false /* !processEvents */);
+    qDeleteAll(d->columns);
+    delete d;
 }
 
 void KDbTableViewData::deleteLater()
 {
-    d->cursor = 0;
+    d->cursor = nullptr;
     QObject::deleteLater();
 }
 
@@ -498,7 +484,7 @@ KDbTableViewColumn* KDbTableViewData::visibleColumn(int index)
 
 bool KDbTableViewData::isDBAware() const
 {
-    return d->cursor != 0;
+    return d->cursor != nullptr;
 }
 
 KDbCursor* KDbTableViewData::cursor() const
@@ -538,7 +524,7 @@ QString KDbTableViewData::dbTableName() const
     return QString();
 }
 
-void KDbTableViewData::setSorting(int column, Qt::SortOrder order)
+void KDbTableViewData::setSorting(int column, KDbOrderByColumn::SortOrder order)
 {
     d->sortOrder = order;
     if (column < 0 || column >= d->columns.count()) {
@@ -548,15 +534,15 @@ void KDbTableViewData::setSorting(int column, Qt::SortOrder order)
     }
     // find proper column information for sorting (lookup column points to alternate column with visible data)
     const KDbTableViewColumn *tvcol = d->columns.at(column);
-    KDbQueryColumnInfo* visibleLookupColumnInfo = tvcol->visibleLookupColumnInfo();
-    const KDbField *field = visibleLookupColumnInfo ? visibleLookupColumnInfo->field : tvcol->field();
+    const KDbQueryColumnInfo* visibleLookupColumnInfo = tvcol->visibleLookupColumnInfo();
+    const KDbField *field = visibleLookupColumnInfo ? visibleLookupColumnInfo->field() : tvcol->field();
     d->sortColumn = column;
     d->realSortColumn = tvcol->columnInfo()->indexForVisibleLookupValue() != -1
                           ? tvcol->columnInfo()->indexForVisibleLookupValue() : d->sortColumn;
 
     // setup compare functor
     d->lessThanFunctor.setColumnType(*field);
-    d->lessThanFunctor.setAscendingOrder(d->sortOrder == Qt::AscendingOrder);
+    d->lessThanFunctor.setSortOrder(d->sortOrder);
     d->lessThanFunctor.setSortColumn(column);
 }
 
@@ -565,7 +551,7 @@ int KDbTableViewData::sortColumn() const
     return d->sortColumn;
 }
 
-Qt::SortOrder KDbTableViewData::sortOrder() const
+KDbOrderByColumn::SortOrder KDbTableViewData::sortOrder() const
 {
     return d->sortOrder;
 }
@@ -609,7 +595,9 @@ bool KDbTableViewData::updateRecordEditBufferRef(KDbRecordData *record,
         int colnum, KDbTableViewColumn* col, QVariant* newval, bool allowSignals,
         QVariant *visibleValueForLookupField)
 {
-    Q_ASSERT(newval);
+    if (!record || !newval) {
+        return false;
+    }
     d->result.clear();
     if (allowSignals)
         emit aboutToChangeCell(record, colnum, newval, &d->result);
@@ -685,7 +673,6 @@ static inline void saveRecordGetValue(const QVariant **pval, KDbCursor *cursor,
 //! @todo if there're multiple views for this data, we need multiple buffers!
 bool KDbTableViewData::saveRecord(KDbRecordData *record, bool insert, bool repaint)
 {
-    Q_UNUSED(record);
     if (!d->pRecordEditBuffer)
         return true; //nothing to do
 
@@ -693,7 +680,7 @@ bool KDbTableViewData::saveRecord(KDbRecordData *record, bool insert, bool repai
     //-check if every NOT NULL and NOT EMPTY field is filled
     QList<KDbTableViewColumn*>::ConstIterator it_f(d->columns.constBegin());
     int col = 0;
-    const QVariant *pval = 0;
+    const QVariant *pval = nullptr;
     QVariant val;
     for (;it_f != d->columns.constEnd() && col < record->count();++it_f, ++col) {
         KDbField *f = (*it_f)->field();
@@ -702,9 +689,9 @@ bool KDbTableViewData::saveRecord(KDbRecordData *record, bool insert, bool repai
             //check it
             if (val.isNull() && !f->isAutoIncrement()) {
                 //NOT NULL violated
-                d->result.msg = tr("\"%1\" column requires a value to be entered.").arg(f->captionOrName())
+                d->result.message = tr("\"%1\" column requires a value to be entered.").arg(f->captionOrName())
                                 + QLatin1String("\n\n") + KDbTableViewData::messageYouCanImproveData();
-                d->result.desc = tr("The column's constraint is declared as NOT NULL.");
+                d->result.description = tr("The column's constraint is declared as NOT NULL.");
                 d->result.column = col;
                 return false;
             }
@@ -713,9 +700,9 @@ bool KDbTableViewData::saveRecord(KDbRecordData *record, bool insert, bool repai
             saveRecordGetValue(&pval, d->cursor, d->pRecordEditBuffer, &it_f, record, f, &val, col);
             if (!f->isAutoIncrement() && (val.isNull() || KDb::isEmptyValue(f->type(), val))) {
                 //NOT EMPTY violated
-                d->result.msg = tr("\"%1\" column requires a value to be entered.").arg(f->captionOrName())
+                d->result.message = tr("\"%1\" column requires a value to be entered.").arg(f->captionOrName())
                                 + QLatin1String("\n\n") + KDbTableViewData::messageYouCanImproveData();
-                d->result.desc = tr("The column's constraint is declared as NOT EMPTY.");
+                d->result.description = tr("The column's constraint is declared as NOT EMPTY.");
                 d->result.column = col;
                 return false;
             }
@@ -727,7 +714,7 @@ bool KDbTableViewData::saveRecord(KDbRecordData *record, bool insert, bool repai
             if (!d->cursor->insertRecord(record, d->pRecordEditBuffer,
                                          d->containsRecordIdInfo /*also retrieve ROWID*/))
             {
-                d->result.msg = tr("Record inserting failed.") + QLatin1String("\n\n")
+                d->result.message = tr("Record inserting failed.") + QLatin1String("\n\n")
                                 + KDbTableViewData::messageYouCanImproveData();
                 KDb::getHTMLErrorMesage(*d->cursor, &d->result);
 
@@ -743,10 +730,10 @@ bool KDbTableViewData::saveRecord(KDbRecordData *record, bool insert, bool repai
             if (!d->cursor->updateRecord(static_cast<KDbRecordData*>(record), d->pRecordEditBuffer,
                                          d->containsRecordIdInfo /*use ROWID*/))
             {
-                d->result.msg = tr("Record changing failed.") + QLatin1String("\n\n")
+                d->result.message = tr("Record changing failed.") + QLatin1String("\n\n")
                                 + KDbTableViewData::messageYouCanImproveData();
 //! @todo set d->result.column if possible
-                KDb::getHTMLErrorMesage(*d->cursor, &d->result.desc);
+                KDb::getHTMLErrorMesage(*d->cursor, &d->result.description);
                 return false;
             }
         }
@@ -774,7 +761,6 @@ bool KDbTableViewData::saveRecord(KDbRecordData *record, bool insert, bool repai
 
 bool KDbTableViewData::saveRecordChanges(KDbRecordData *record, bool repaint)
 {
-    Q_UNUSED(record);
     d->result.clear();
     emit aboutToUpdateRecord(record, d->pRecordEditBuffer, &d->result);
     if (!d->result.success)
@@ -789,7 +775,6 @@ bool KDbTableViewData::saveRecordChanges(KDbRecordData *record, bool repaint)
 
 bool KDbTableViewData::saveNewRecord(KDbRecordData *record, bool repaint)
 {
-    Q_UNUSED(record);
     d->result.clear();
     emit aboutToInsertRecord(record, &d->result, repaint);
     if (!d->result.success)
@@ -804,7 +789,6 @@ bool KDbTableViewData::saveNewRecord(KDbRecordData *record, bool repaint)
 
 bool KDbTableViewData::deleteRecord(KDbRecordData *record, bool repaint)
 {
-    Q_UNUSED(record);
     d->result.clear();
     emit aboutToDeleteRecord(record, &d->result, repaint);
     if (!d->result.success)
@@ -813,8 +797,8 @@ bool KDbTableViewData::deleteRecord(KDbRecordData *record, bool repaint)
     if (d->cursor) {//db-aware
         d->result.success = false;
         if (!d->cursor->deleteRecord(static_cast<KDbRecordData*>(record), d->containsRecordIdInfo /*use ROWID*/)) {
-            d->result.msg = tr("Record deleting failed.");
-            //! @todo use KDberrorMessage() for description (desc) as in KDbTableViewData::saveRecord() */
+            d->result.message = tr("Record deleting failed.");
+            //! @todo use KDberrorMessage() for description as in KDbTableViewData::saveRecord() */
             KDb::getHTMLErrorMesage(*d->cursor, &d->result);
             d->result.success = false;
             return false;
@@ -854,7 +838,6 @@ void KDbTableViewData::deleteRecords(const QList<int> &recordsToDelete, bool rep
 
 void KDbTableViewData::insertRecord(KDbRecordData *record, int index, bool repaint)
 {
-    Q_UNUSED(record);
     insert(index = qMin(index, count()), record);
     emit recordInserted(record, index, repaint);
 }
@@ -891,7 +874,7 @@ bool KDbTableViewData::deleteAllRecords(bool repaint)
     return res;
 }
 
-int KDbTableViewData::autoIncrementedColumn()
+int KDbTableViewData::autoIncrementedColumn() const
 {
     if (d->autoIncrementedColumn == -2) {
         //find such a column
@@ -946,3 +929,24 @@ QString KDbTableViewData::messageYouCanImproveData()
 {
     return tr("Please correct data in this record or use the \"Cancel record changes\" function.");
 }
+
+QDebug operator<<(QDebug dbg, const KDbTableViewData &data)
+{
+    dbg.nospace() << "TableViewData(";
+    dbg.space() << "sortColumn:" << data.sortColumn()
+                << "sortOrder:" << (data.sortOrder() == KDbOrderByColumn::SortOrder::Ascending
+                                    ? "asc" : "desc")
+                << "isDBAware:" << data.isDBAware()
+                << "dbTableName:" << data.dbTableName()
+                << "cursor:" << (data.cursor() ? "yes" : "no")
+                << "columnCount:" << data.columnCount()
+                << "count:" << data.count()
+                << "autoIncrementedColumn:" << data.autoIncrementedColumn()
+                << "visibleColumnCount:" << data.visibleColumnCount()
+                << "isReadOnly:" << data.isReadOnly()
+                << "isInsertingEnabled:" << data.isInsertingEnabled()
+                << "containsRecordIdInfo:" << data.containsRecordIdInfo()
+                << "result:" << data.result();
+    dbg.nospace() << ")";
+    return dbg.space();
+}
